"""Modul to with function to work with the table configuration"""
import logging
import os
import json
from ringo.lib.helpers import get_path_to, get_app_inheritance_path
from ringo.lib.cache import CACHE_TABLE_CONFIG

log = logging.getLogger(__name__)


def get_table_config(clazz, tablename=None):
    """Returns the table (overview, listing) configuration with the
    name 'tablename' of this Item from the configuration file. If
    the default table configuration will be returned. The table
    configuration is cached for later requests.

    :clazz: @todo
    :tablename: @todo
    :returns: @todo

    """
    # As this is a class method of the BaseItem we need to build a
    # unique cachename for tableconfigs among all inherited classes.
    cachename = "%s.%s" % (clazz.__name__, tablename)
    if not CACHE_TABLE_CONFIG.get(cachename):
        CACHE_TABLE_CONFIG.set(cachename, TableConfig(clazz, tablename))
    return CACHE_TABLE_CONFIG.get(cachename)


def get_path_to_overview_config(filename, app=None, location=None):
    """Returns the path the the given overview configuration. The file name
    should be realtive to the default location for the configurations.

    :file: filename
    :returns: Absolute path to the configuration file

    """
    if location is None:
        location = "views/tables"
    return get_path_to(os.path.join(location, filename), app)


class TableConfig:
    """The TableConfig clazz provides an interface for configuring the
    overview of the the given clazz. The configuration includes

    * Enabled features of the overview
    * Which fields should be enabled in the overview
    * Labeling of the fields
    * Rendering of the fields
    * Layout of the table (column width etc.)

    The configuration of the overview is loaded from a an JSON
    configuration file which is located under view/overviews relative to
    the application root.

    The configuration has the following form and options::

        {
            "overview": {
                "columns": [
                    {
                        "name": "fieldname",
                        "label": "Label",
                        "width": "width",
                        "screen": "xlarge",
                        "expand": true,
                        "filter": false,
                        "title": "Tooltip title"
                    }
                ]
                "settings": {
                    "default-sort-field": "name",
                    "default-sort-order": "desc",
                    "auto-responsive": true,
                    "advancedsearch": true
                },
                "filters": [
                    {
                        "field": "fieldname",
                        "type": "checkbox",
                        "expr": "",
                        "regex": false,
                        "smart": true,
                        "caseinsensitive": true
                    }
                ]
            }
        }

    The configuration can have configurations for more than one table
    configuration. The default name of a configuration is *overview*.
    The next sections defines the *columns* of the table. The columns
    are rendered in the order they are defined. Each column can have the
    following options:

    * *name*: The name of the attribute within clazz which should be
      listed here.
    * *label*: The label of the field.
    * *width*: The width of the field. If not units are given the pixel
      are assumed.
    * *screen*: Define from which size on this field will be displayed.
      Defaults to "small" which means rendering on all sizes.
      Available media sizes: (xlarge, large, medium, small).
    * *expand*: The expand option is used to expand the referneces
      values in selections into the literal value of the corrispondig
      option. Note that this option is only usefull for selection fields
      in *formbar* which do not have a real relation attached. In all
      other cases the reference values are expanded automatically.
    * *filter*: The filter determines if the rendered link is a filter
      link. A filter link is a shortcut to trigger a search with the
      current value of the field in the current column over all items.
      Defaults to No.
    * *title* A tooltip will be rendered for the table header of the
      column.
    * *roles* A comma separated list of rolenames. If defined the column
      will only be listed for users which have the given role. Default
      behaviour is to list a columns to all roles.

    Further the table has some table wide configuration options:

    * *sortable*: If True, the table is sortable.
    * *default-sort-field*: Name of the column which should be used as
      default sorting on the table. Defaults to the first column in the table.
    * *default-sort-order*: Sort order (desc, asc) Defaults to asc.

    * *searchable*: If True, the table is searchable and a search field is
      shown.
    * *default-search*: Default search filter for the table
    * *advancedsearch*: If False, a more complex overview with stacked
      search regex and save features is used. Otherwise a more simple
      overview. Default to use the simple overview.

    * *pagination*: If True pagination of the results will be enabled.
      The table will have gui element to configure pagination of the
      table. Defaults to false.

    * *auto-responsive*: If True than only the first column of a table
      will be displayed on small devices. Else you need to configure the
      "screen" attribute for the fields.
    * *show-info*: It True than a info field showing number of items in the 
      table

    For DT tables you can define different search filters which are
    defined in the *filters* section. Filters is a list of different
    filter options. Each filter has the following options:

    * *field*: The name of the field on which the filter will be applied.
    * *label*: The label of the filter.
    * *active*: If true the filter will be active.
    * *type*: Type of the rendering. Currently only "checkbox" is
      supported. The checkbox renderer will active or deactive a filter
      a a certain row.
    * *expr*: The expression for the filter.
    * *smart*: If set to True this filter is a smart filter. Defaults to True.
    * *regex*: If set to True this the expression is handled as a
      regular expression. Defaults to False.
    * *caseinsensitive*: If set to True the search is caseinsensitive.
      Defaults to True.


    """

    def __init__(self, clazz, name):
        """Will initialize the configuration of the overview for the
        clazz.

        :clazz: The clazz for which the overview configuration should be
        loaded.
        :name: name of the configuration. Defaults to the
        "overview" configuration.
        """
        self.clazz = clazz
        self.name = name or "overview"
        self.config = _load_overview_config(clazz)

    def get_settings(self):
        """Returns the settings for the table as dictionary
        :returns: Settings dictionary

        """
        config = self.config.get(self.name)
        return config.get('settings', {})

    def get_filters(self):
        """Returns the filters for the table as dictionary
        :returns: Filters dictionary

        """
        config = self.config.get(self.name)
        return [Filter(fltr) for fltr in config.get('filters', [])]

    def is_autoresponsive(self):
        settings = self.get_settings()
        return settings.get("auto-responsive", True)

    def is_bundled(self):
        settings = self.get_settings()
        return settings.get("bundled", True)

    def is_sortable(self):
        settings = self.get_settings()
        return settings.get("sortable", True)

    def is_searchable(self):
        settings = self.get_settings()
        return settings.get("searchable", True)

    def is_paginated(self):
        settings = self.get_settings()
        return settings.get("pagination", False)

    def is_advancedsearch(self, default=False):
        settings = self.get_settings()
        return settings.get("advancedsearch", default)

    def get_columns(self, user=None):
        """Return a list of configured columns within the configuration.
        Each colum is a dictionary containing the one or more available
        conifguration attributes."""
        from ringo.lib.security import has_role
        cols = []
        config = self.config.get(self.name)
        for col in config.get('columns'):
            if user and col.get("roles"):
                # Check if user has on of the required roles.
                roles = col.get("roles").split(",") + ['admin']
                for role in roles:
                    if has_role(user, role):
                        cols.append(col)
                        break
            else:
                cols.append(col)
        return cols

    def get_column_index(self, name):
        """Will return the index of the column in the overview. Index is
        count from left to right. This method is used to match the
        configured columns in this configuration with the columns in the
        datatables which can be addressed by its index. Please note the
        the index of the datatables columns are increased by 1. So you
        will need to add 1 to the result of this method."""
        for num, col in enumerate(self.get_columns()):
            if col["name"] == name:
                return num
        return 0

    def get_default_sort_column(self):
        """Returns the name of the attribute of the clazz which is
        marked as field for default sorting in the *settings* section of
        the configuration. If no default sorting is configured then
        return the name of the attribute in the first column which is
        configured in the table"""
        settings = self.get_settings()
        if settings:
            def_sort = settings.get('default-sort-field')
            if def_sort:
                return def_sort
        return self.get_columns()[0].get('name')

    def get_default_sort_order(self):
        """Returns the ordering of the sort in the table """
        settings = self.get_settings()
        if settings:
            def_order = settings.get('default-sort-order')
            if def_order:
                return def_order
        return "asc"

    def get_default_search(self):
        """Returns the seaech filter of the items in the table """
        settings = self.get_settings()
        if settings:
            def_search = settings.get('default-search')
            if def_search:
                return def_search
        return []


def _load_overview_config(clazz):
    """Return a datastructure representing the overview
    configuration. The configuration is loaded from a JSON
    configuration file. The function will first try to load the
    application specific configuration. If this fails it will try to
    load it form the extension specific loaction or orign application
    and finally from ringo.  If no configuration can be found an
    exception is raised."""
    cfile = "%s.json" % clazz.__tablename__
    config = None
    name = clazz.__module__.split(".")[0]
    for appname in get_app_inheritance_path():
        try:
            # Always first try to load from the current application. No
            # matter what the current name is as name can be different from
            # the appname in case of loading forms for an extension. In this
            # case first try to load the form configuration from the
            # application to be able to overwrite the forms.
            config = open(get_path_to_overview_config(cfile, appname), "r")
            break
        except IOError:
	    # Silently ignore IOErrors here as is Ok when trying to load the
	    # configurations files while iterating over the possible config
	    # file locations. If the file can finally not be loaded an IOError
	    # is raised at the end.
            pass
    else:
        if name.startswith("ringo_"):
            config = open(get_path_to_overview_config(cfile,
                                                      name,
                                                      location="."), "r")
    # If we can't load the config file after searching in all locations, raise
    # an IOError. Hint: Maybe you missed to set the app.base config variable?
    if not config:
        raise IOError("Could not load table configuration for %s" % cfile)
    return json.load(config)


class Filter(object):
    """This class represents a custom filter used in Listings using the
    datatables JS extension. This class is used to give access to the
    relevant attribute of a filter. Some of the attributes are processed
    to be ready to be used directly in JS code."""

    def __init__(self, conf):
        self._conf = conf
        self.expr = self._conf.get("expr", "")
        self.field = self._conf.get("field", "")
        self.label = self._conf.get("label", "")
        self.type = self._conf.get("type", "checkbox")
        self.active = self._conf.get("active", True)

        # JS Options used in filter method
        if self._conf.get("regex", False):
            self.regex = "true"
        else:
            self.regex = "false"
        if self._conf.get("smart", True) and not self._conf.get("regex", False):
            self.smart = "true"
        else:
            self.smart = "false"
        if self._conf.get("caseinsensitive", True):
            self.caseinsensitive = "true"
        else:
            self.caseinsensitive = "false"



import logging
import cgi
import os
import pkg_resources
from mako.lookup import TemplateLookup
from formbar.renderer import (
    FieldRenderer,
    DropdownFieldRenderer as FormbarDropdown,
    SelectionFieldRenderer as FormbarSelectionField,
    CheckboxFieldRenderer as FormbarCheckboxField
)
import ringo.lib.helpers as helpers
from ringo.lib.helpers import get_action_routename, literal, escape, HTML
from ringo.model.base import BaseItem, BaseList, get_item_list
from ringo.lib.table import get_table_config
import ringo.lib.security as security

base_dir = pkg_resources.get_distribution("ringo").location
template_dir = os.path.join(base_dir, 'ringo', 'templates')
template_lookup = TemplateLookup(directories=[template_dir],
                                 default_filters=['h'])

log = logging.getLogger(__name__)

###########################################################################
#                         Renderers for form elements                     #
###########################################################################

def add_renderers(custom_renderers):
    """Helper function to add ringo ringo specific renderers for form
    rendering."""
    for key in custom_renderers:
        if not key in renderers:
            renderers[key] = custom_renderers[key]
    return renderers


def get_link_url(item, request, actionname=None, backurl=False):
    """Return a url to the given item. On default the link will be a
    link to the update or read view of the item depending on the
    permission. If the user does not have enough permissions than None
    is returned. Optionally you can provide the name of a action. In
    this case the url will be build for the given actionname if the user
    has enough permissions."""
    if isinstance(item, BaseItem):
        if actionname:
            from ringo.views.helpers import get_item_modul
            modul = get_item_modul(request, item)
            action = modul.get_action(actionname)
            if action is None:
                # This can happen if the action is not part of the modul
                # but a custum userdefined view. No permission checks
                # are done here yet.
                route_name = get_action_routename(item, actionname)
            else:
                permission = action.permission or action.name.lower()
                if security.has_permission(permission, item, request):
                    route_name = get_action_routename(item,
                                                      action.name.lower())
                else:
                    return None
        elif security.has_permission("update", item, request):
            route_name = get_action_routename(item, 'update')
        elif security.has_permission("read", item, request):
            route_name = get_action_routename(item, 'read')
        else:
            return None

        query = {}
        if backurl:
            query['backurl'] = request.current_route_path()
        return request.route_path(route_name, id=item.id, _query=query)
    return None


def filter_options_on_permissions(request, options):
    """Will filter the given options based on the permissions on the
    current user of the request. After filtering the options will only
    have items where the user is allowed to at least read it.

    :request: current request
    :options: list of tuple of options (item, value, filtered)
    :returns: filtered list of option tuples.

    """
    filtered_options = []
    for option in options:
        visible = False
        if (option[2] and (security.has_permission('read', option[0], request)
           or not hasattr(option[0], 'owner'))):
            visible = True
        filtered_options.append((option[0], option[1], visible))
    return filtered_options


class LinkFieldRenderer(FieldRenderer):
    """ Will rendere a Link to the item. The item which will be linked
    can be set by

    * the name attribute of the entity
    * the value attribute of the entity

    If the item can not be determined by one of the options above the
    item will be the item linked to the form.

    The following option in
    addition to the base Linkfield Renderer are supported:

    * openmodal: If true the item will be opened in a modal form.
      Defaults to false. If true a backlink will be generated too.
    * action: Name of the action for which the link will be rendered.
      Defaults to None with means the link will be either to the read or
      update view.
    * css: CSS classes for the link.
    * backlink: If set to true a backlink will be generated.
    * target: Set target attribute of the link.
    """
    def __init__(self, field, translate):
        """@todo: to be defined"""
        FieldRenderer.__init__(self, field, translate)
        self.template = template_lookup.get_template("internal/linkfield.mako")

    def _get_template_values(self):
        values = FieldRenderer._get_template_values(self)
        values['link_text'] = self._field.label
        try:
            item = getattr(self._field._form._item, self._field.name)
        except AttributeError:
            # If the attribute is not part of the item there may be a
            # value in the field based on a expression. So set the item
            # to the value of the field. If this is not an instance of
            # BaseItem the link can not be generated.
            item = self._field.value
            if not item:
                name = self._field.name
                item = self._field._form._item
                # Only print warning if the fieldname is not a helper
                # field beginning with "_"
                if not name.startswith("_"):
                    log.warning("Missing value for %s in %s" % (name, item))
        backurl = self.openmodal == "true" or self.backlink == "true"
        values['url'] = get_link_url(item,
                                     self._field._form._request,
                                     self._field.renderer.action,
                                     backurl) or "#"
        return values

    def _render_label(self):
        return ""


class CheckboxFieldRenderer(FormbarCheckboxField):
    """Ringo specific DropdownFieldRenderer. Will additionally check
    permission to read on the items in the option list"""

    def _get_template_values(self):
        values = FormbarCheckboxField._get_template_values(self)
        values['options'] = filter_options_on_permissions(
            self._field._form._request,
            values['options'])
        return values


class DropdownFieldRenderer(FormbarDropdown):
    """Ringo specific DropdownFieldRenderer. This renderer add a small
    link next to the label to make it possible to jump to the selected
    item in the dropdown list.

    * nolink: Flag "true" or "false" to configure completly disable
      linking Default to "false".

    """

    def __init__(self, field, translate):
        """@todo: to be defined"""
        FormbarDropdown.__init__(self, field, translate)
        self.template = template_lookup.get_template("internal/dropdown.mako")

    def _get_template_values(self):
        values = FormbarDropdown._get_template_values(self)
        values['options'] = filter_options_on_permissions(
            self._field._form._request,
            values['options'])
        return values

    def render_link(self):
        html = []
        items = []
        try:
            item = getattr(self._field._form._item, self._field.name)
        except AttributeError:
            log.warning("Missing %s attribute in %s"
                        % (self._field.name, self._field._form._item))
            return literal("").join(html)

        if not isinstance(item, list):
            items.append(item)
        else:
            items = item
        for item in items:
            url = get_link_url(item, self._field._form._request)
            if url:
                html.append(HTML.tag("a", href=("%s" % url), _closed=False))
                html.append(escape(unicode(item)))
                html.append(HTML.tag("/a", _closed=False))
        return literal("").join(html)

    def _render_label(self):
        html = []
        html.append(FormbarDropdown._render_label(self))
        if not self._field.is_readonly() and not self.nolink == "true":
            link = self.render_link()
            if link:
                html.append(" [")
                html.append(link)
                html.append("]")
        return literal("").join(html)


class StateFieldRenderer(FormbarDropdown):
    """Ringo specific DropdownFieldRenderer to change the current state
    of an item.  This renderer will render a combined widget containing
    a textfield showing the current state of the item and a dropdown
    with available actions which can be done from this state.

    * layout: Option to change the layout of the statefield. The
    renderer offers currently three options to render setting the
    `layout` attribute of the renderer:

        1. default: In default the renderer shows the current state with
        description and the resulting state and description when
        choosing a transtion.
        2. simple: Simple will render a simple dropdown with the current
        state as part of the fields label and the available transitions
        as options of the dropdown
        3. button: Botton will render the current state the available
        transitions as a button. The button will work like a submit
        button of the form so if the user clicks on it the form data
        will be submitted and the state is changed.

    """

    def __init__(self, field, translate):
        """@todo: to be defined"""
        FormbarDropdown.__init__(self, field, translate)
        self.template = template_lookup.get_template("internal/"
                                                     "statefield.mako")

    def _render_label(self):
        return literal("")

    def render(self):
        """Initialize renderer"""
        html = []
        # Get all available transitions from the current state for this
        # item and request.
        item = self._field._form._item
        sm = item.get_statemachine(self._field.name,
                                   request=self._field._form._request)
        state = sm.get_state()
        has_errors = len(self._field.get_errors())
        has_warnings = len(self._field.get_warnings())

        class_options = "form-group %s %s" % ((has_errors and 'has-error'),
                                              (has_warnings and 'has-warning'))
        #html.append(HTML.tag("div", _closed=False,
        #                     class_=class_options))
        html.append(self._render_label())
        values = {'field': self._field,
                  'request': self._field._form._request,
                  'state': state,
                  '_': self._field._form._translate}
        html.append(literal(self.template.render(**values)))
        #html.append(HTML.tag("\div", _closed=False))
        return literal("").join(html)


class ListingFieldRenderer(FormbarSelectionField):
    """Renderer to render a listing of linked items. Used attributes:

    * form: Name of the form which is used to add new items
    * table: Name of the table configuration which is used to list items
    * hideadd: Flag "true" or "false" to configure hiding the add button.
    * nolinks: Flag "true" or "false" to configure completly disable
      linking.
    * showsearch: Flag "true" or "false" to configure rendering a search
      field.
    * showall: "true" or "false". If true all items (linked and
      unlinked) regardless if the current user is allowed to read
      or update the item will be listed. However clicking on items the
      user has no permission to read will have no effect as no links are
      rendered for those items. Defaults to false.
    * onlylinked: "true" or "false". If true only linked items will be
      rendered. Checkboxes will be hidden.
    * multiple: "true" or "false". If false only one option can be
      selected. Defaults to true. Note that this restriction is only
      implemented on client side.
    * openmodal: "true" or "false". If true the item will be opened in a
      modal popup.
    * backlink: "true" or "false". If true the user will be redirected
      back to the listing after creating a new item. Defaults to true.

    Example::

        <entity ...>
            <renderer type="listing" showall="true" table="details"/>
        </entity>
    """

    def __init__(self, field, translate):
        FormbarSelectionField.__init__(self, field, translate)
        self.template = template_lookup.get_template("internal/listfield.mako")

    def get_class(self):
        try:
            clazz = self._field._get_sa_mapped_class()
            return clazz
        except AttributeError:
            # Special logic if the fields item is not an SQLAlchemy
            # attribute but a python property (used to add futher
            # informations to the fields item model). In this case getting
            # the clazz from the SA attributes mapper -> clazz will
            # fail.
            clazz = getattr(self._field._form._item, self._field.name)
            if isinstance(clazz, BaseList):
                return clazz.clazz
            return clazz

    @property
    def itemlist(self):
        clazz = self.get_class()
        itemlist = get_item_list(self._field._form._request, clazz)
        config = get_table_config(itemlist.clazz,
                                  self._field._config.renderer.table)
        sort_field = config.get_default_sort_column()
        sort_order = config.get_default_sort_order()
        itemlist.sort(sort_field, sort_order)

        # Warning filtering items here can cause loosing relations to
        # the filtered items. This is esspecially true if the item which
        # was related to the item before now gets filtered because of a
        # changes value in an attribute e.g. In this case the filtered
        # item is not in the list at all and will not be sent on a POST
        # request. This will result in removing the relation!
        search  = config.get_default_search()
        itemlist.filter(search)
        return itemlist

    def _get_selected_items(self, items):
        try:
            selected = getattr(self._field._form._item, self._field.name) or []
            # Usually the getattr function should return a single item
            # or a list of items. But in some cases the attribute can be
            # a BaseList. This can happen if the attribute is a python
            # property of the item to dynamically add some attributes.
            # This property might return a BaseList.
            if isinstance(selected, BaseList):
                selected = selected.items
            elif not isinstance(selected, list):
                selected = [selected]
            selected_ids = [s.id for s in selected]
            return [i for i in items if i.id in selected_ids]
        except AttributeError:
            # Can happen when designing forms an the model of the item
            # is not yet configured.
            log.warning("Missing %s attribute in %s"
                        % (self._field.name, repr(self._field._form._item)))
        return []

    def render(self):
        """Initialize renderer"""
        html = []
        active = 'active' if self._active else 'inactive'
        config = self._field._config.renderer
        has_errors = len(self._field.get_errors())
        has_warnings = len(self._field.get_warnings())
        class_options = "form-group %s %s %s" % ((has_errors and 'has-error'),
                                              (has_warnings and 'has-warning'),(active))
        html.append(HTML.tag("div", _closed=False,
                             rules=u"{}".format(";".join(self._field.rules_to_string)),
                             formgroup="{}".format(self._field.name),
                             desired="{}".format(self._field.desired),
                             required="{}".format(self._field.required),
                             class_=class_options))
        html.append(self._render_label())
        if self._field.is_readonly() or self.onlylinked == "true":
            items = self._get_selected_items(self.itemlist.items)
        else:
            items = self.itemlist.items

        # Get filtered options and only use the items which are
        # in the origin items list and has passed filtering.
        items = self._field.filter_options(items)
        # Now filter the items based on the user permissions
        if self.showall != "true": 
            items = filter_options_on_permissions(self._field._form._request,
                                                  items)

        values = {'items': items,
                  'field': self._field,
                  'clazz': self.get_class(),
                  'pclazz': self._field._form._item.__class__,
                  'request': self._field._form._request,
                  '_': self._field._form._translate,
                  's': security,
                  'h': helpers,
                  'tableconfig': get_table_config(self.itemlist.clazz,
                                                  config.table)}
        html.append(literal(self.template.render(**values)))
        html.append(self._render_errors())
        html.append(self._render_help())
        html.append(HTML.tag("/div", _closed=False))
        return literal("").join(html)

renderers = {
    "dropdown": DropdownFieldRenderer,
    "checkbox": CheckboxFieldRenderer,
    "listing": ListingFieldRenderer,
    "state": StateFieldRenderer,
    "link": LinkFieldRenderer
}



"""Rename gid to default_gid in usergroups and modules

Revision ID: 3520cf26d8d8
Revises: 4ff2bc4eb9bf
Create Date: 2015-07-21 22:51:53.672633

"""

# revision identifiers, used by Alembic.
revision = '3520cf26d8d8'
down_revision = '4ff2bc4eb9bf'

from alembic import op
import sqlalchemy as sa


UPGRADE = """
"""
DOWNGRADE = """
"""


def iter_statements(stmts):
    for st in [x for x in stmts.split('\n') if x]:
        op.execute(st)


def upgrade():
    ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table("users") as batch_op:
        batch_op.add_column(sa.Column('default_gid', sa.Integer(), nullable=True))
        batch_op.create_foreign_key('fk_defaul_gid_usergroups', 'usergroups', ['default_gid'], ['id'])

    with op.batch_alter_table("modules") as batch_op:
        batch_op.add_column(sa.Column('default_gid', sa.Integer(), nullable=True))
        batch_op.create_foreign_key('fk_defaukt_gid_usergroups', 'usergroups', ['default_gid'], ['id'])
    iter_statements(UPGRADE)

    with op.batch_alter_table("modules") as batch_op:
        batch_op.drop_column('gid')
    with op.batch_alter_table("users") as batch_op:
        batch_op.drop_column('gid')
    ### end Alembic commands ###


def downgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.add_column('users', sa.Column('gid', sa.INTEGER(), autoincrement=False, nullable=True))
    op.add_column('modules', sa.Column('gid', sa.INTEGER(), autoincrement=False, nullable=True))
    iter_statements(DOWNGRADE)
    op.drop_column('users', 'default_gid')
    op.drop_column('modules', 'default_gid')
    ### end Alembic commands ###



"""Modul with helper functions to work with sqlalchemy."""
from sqlalchemy.orm import ColumnProperty, class_mapper


def get_columns_from_clazz(clazz, include_relations=False):
    return [prop.key for prop in class_mapper(clazz).iterate_properties
            if isinstance(prop, ColumnProperty) or include_relations]


def get_columns_from_instance(item, include_relations=False):
    return get_columns_from_clazz(item.__class__, include_relations)


def get_relations_from_clazz(clazz):
    only_columns = set(get_columns_from_clazz(clazz))
    with_relations = set(get_columns_from_clazz(clazz, True))
    return with_relations-only_columns



"""Modul to get information about the application"""
import os
import pkg_resources
from pyramid.threadlocal import get_current_registry
from ringo.lib.sitetree import build_breadcrumbs, site_tree_branches


def get_ringo_version():
    return pkg_resources.get_distribution('ringo').version


def get_app_inheritance_path():
    """Returns a list of application names. The names describe the path
    to the root of the application inheritance. e.g if the current
    application is 'foo' which is based and 'bar' which is based on
    'ringo' the function will return the follwing result: ['foo', 'bar',
    'ringo'].

    The default path is [<nameofcurrentapp>, "ringo"]. The path can be
    extended by setting the app.base config variable.

    :returns: List of application name which build the inheritance path.
    """
    path = ['ringo']
    registry = get_current_registry()
    settings = registry.settings
    base = settings.get("app.base")
    if base:
        path.append(base)
    path.append(get_app_name())
    return reversed(path)


def get_app_name():
    registry = get_current_registry()
    return registry.__name__


def get_app_version():
    return pkg_resources.get_distribution(get_app_name()).version


def get_app_location(name=None):
    if not name:
        name = get_app_name()
    return pkg_resources.get_distribution(name).location


def get_app_url(request):
    """Returns the path of the application under which the application
    is hosted on the server.

    .. note::
        This function is a helper function. It is only used to build
        correct URLs for client sided AJAX requests in case the
        application is hosted in a subpath.

    Example:

    If the application is hosted on "http://localhost:6543/foo" the
    function will return "foo". If it is hosted under the root
    directory '' is returned."""
    return request.environ.get("SCRIPT_NAME", "")


def get_app_mode(request):
    """Will return a tuple of the mode configuration (if configured)

    Tuple: (mode, desc, color)

    If no mode is configured return None.

    :request: Current request
    :return: Tuple of mode configruation
    """
    settings = request.registry.settings
    mode = settings.get("app.mode")
    desc = settings.get("app.mode_desc", "").decode('utf-8')
    color_primary = settings.get("app.mode_color_primary", "#F2DEDE")
    color_secondary = settings.get("app.mode_color_secondary", "red")
    if mode:
        return (mode, desc, color_primary, color_secondary)
    return None


def get_app_title():
    """Will return the title of the application

    :return: The title of the application"""
    registry = get_current_registry()
    settings = registry.settings
    return settings['app.title']


def get_path_to(location, app=None):
    """Will return the full pathname the given file name with in the
    path. path is relativ to the application package (pkg_ressource
    location + ressource name). You can define a alternative
    application."""
    if app:
        app_name = app
    else:
        app_name = get_app_name()
    base_path = os.path.join(get_app_location(app_name), app_name)
    return os.path.join(base_path, location)


def get_breadcrumbs(request, strategy=None):
    """Will return a list of elements which are used to build the
    breadcrumbs in the UI.

    The function take a strategy attribute which is called to build this
    list instead of the default mechanism of ringo. The strategy
    function takes the current request as attribute

    The returned list currently must have the follwing format::

        [(label of element, url of element), (), ...]

    The last element in the list shoul be the current element and has no
    link. (URL is None)

    :request: Current request
    :strategy: Optional function which is called to build the site tree.
    :returns: List of elements used for building a the breadcrumbs.

    """
    if strategy is None:
        strategy = build_breadcrumbs
    tree = {}
    for branch in site_tree_branches:
        tree.update(branch)
    return strategy(request, tree)



import os
import fnmatch
import json
import subprocess
import uuid
import transaction
import pkg_resources
from sqlalchemy import func
from mako.lookup import TemplateLookup
from ringo.lib.helpers import get_app_location, dynamic_import
from ringo.scripts.db import (
    get_session, create_new_revision
)
from ringo.model.modul import ModulItem, ActionItem

# Directory with templates to generate views and models
base_dir = pkg_resources.get_distribution("ringo").location
modul_template_dir = os.path.join(base_dir, 'ringo', 'scripts', 'templates')
template_lookup = TemplateLookup(directories=[modul_template_dir],
                                 default_filters=['h'])

mixinmap = {
    'owned': 'Owned',
    'meta': 'Meta',
    'state': 'StateMixin',
    'blobform': 'Blobform',
    'versioned': 'Versioned',
    'nested': 'Nested'
}


def get_modul_fixture(name, package, session):
    id = get_next_modulid(package, session)
    fixture = {}
    fixture["id"] = id
    fixture["name"] = name + "s"
    fixture["label"] = name.capitalize()
    fixture["label_plural"] = fixture["label"] + "s"
    fixture["clazzpath"] = ".".join([package, 'model',
                                     name, fixture["label"]])
    fixture["display"] = "header-menu"
    fixture["str_repr"] = "%s|id"
    fixture["uuid"] = str(uuid.uuid4())
    return fixture


def get_action_fixtures(session, mid, ignore=[]):
    # TODO: Translate the name of the Action (torsten) <2013-07-10 09:32>
    action_id = get_next_actionid(session)
    fixtures = []
    for action in ['list', 'create', 'read', 'update',
                   'delete', 'import', 'export']:
        if action in ignore:
            continue
        fixture = {}
        myuuid = str(uuid.uuid4())
        if action == "list":
            name = "List"
            url = "list"
            icon = "icon-list-alt"
            bundle = False
        if action == "create":
            name = "Create"
            url = "create"
            icon = "icon-plus"
            bundle = False
        if action == "read":
            name = "Read"
            url = "read/{id}"
            icon = "icon-eye-open"
            bundle = False
        if action == "update":
            name = "Update"
            url = "update/{id}"
            icon = "icon-edit"
            bundle = False
        if action == "delete":
            name = "Delete"
            url = "delete/{id}"
            icon = "icon-eye-delete"
            bundle = False
        if action == "import":
            name = "Import"
            url = "import"
            icon = "icon-import"
            bundle = False
        if action == "export":
            name = "Export"
            url = "export/{id}"
            icon = "icon-export"
            bundle = True

        fixture["id"] = action_id
        fixture["mid"] = mid
        fixture["name"] = name
        fixture["description"] = ""
        fixture["permission"] = ""
        fixture["url"] = url
        fixture["icon"] = icon
        fixture["uuid"] = myuuid
        fixture["bundle"] = bundle

        fixtures.append(fixture)
        action_id += 1
    return fixtures


def remove_db_entry(name, session):
    print 'Remove entry in modules table for "%s"... ' % name,
    modul_name = name + "s"
    try:
        with transaction.manager:
            session.query(ModulItem).filter(
                ModulItem.name == modul_name).delete()
            session.flush()
        # Get last inserted id.
        print 'Ok.'
    except Exception, e:
        print e
        print 'Failed.'


def get_next_modulid(package, session):
    if package == "ringo":
        id = session.query(
            func.max(ModulItem.id)).filter(ModulItem.id < 1000).one()[0]
    else:
        id = session.query(
            func.max(ModulItem.id)).filter(ModulItem.id > 999).one()[0]
    if id:
        return id + 1
    else:
        return 1000


def get_next_actionid(session):
    id = session.query(func.max(ActionItem.id)).one()[0]
    return id + 1


def _get_fixture_file(path, pattern):
    for root, dirs, files in os.walk(path):
        for name in files:
            if fnmatch.fnmatch(name, pattern):
                return os.path.join(root, name)

def replace_sql_in_revision(path, package, name, session):
    content = None
    sql = []
    modul_fixture = get_modul_fixture(name, package, session)
    modul_sql = ("INSERT INTO modules (id, name, description, label, ",
                 "label_plural, clazzpath, display, str_repr, uuid) ",
                 "VALUES ({id}, '{name}', '', '{label}', '{label_plural}', ",
                 "'{clazzpath}', '{display}', '{str_repr}', '{uuid}');")
    sql.append("".join(modul_sql).format(**modul_fixture))
    modul_id = modul_fixture["id"]
    actions_sql = []
    for fixture in get_action_fixtures(session, modul_id, ignore=[]):
        action_sql = ("INSERT INTO actions (id, mid, uuid, name, url, ",
                      "icon, bundle, description, display, permission) ",
                      "VALUES ({id}, {mid}, '{uuid}', '{name}', ",
                      "'{url}', '{icon}', '{bundle}', '', '', '');")
        actions_sql.append("".join(action_sql).format(**fixture))
    sql.append("\n".join(actions_sql))
    with open(path, "r") as rf:
        print 'Adding data to revision "%s"... ' % path,
        content = rf.read()
        content = content.replace('INSERTS = """"""',
                                  'INSERTS = """\n%s\n"""' % "\n".join(sql))

    with open(path, "r+") as rf:
        rf.seek(0)
        rf.write(content)

    print 'Ok'

def add_fixtures(package, name, session):
    """Will add the fixtures for the new modul to the fixture files.

    :package: @todo
    :name: @todo
    :session: @todo
    :returns: @todo

    """
    path = os.path.join(get_app_location(package), package, 'fixtures')
    modul_fixture = get_modul_fixture(name, package, session)
    modul_file = _get_fixture_file(path, "*_modules.json")

    with open(modul_file, "r+") as mf:
        print 'Adding data to fixture "%s"... ' % modul_file,
        modul_json = json.load(mf)
        modul_json.append(modul_fixture)
        mf.seek(0)
        mf.write(json.dumps(modul_json, indent=4))
        print 'Ok'

    modul_id = modul_fixture["id"]
    action_fixtures = get_action_fixtures(session, modul_id)
    action_file = _get_fixture_file(path, "*_actions.json")

    with open(action_file, "r+") as af:
        print 'Adding data to fixture "%s"... ' % action_file,
        action_json = json.load(af)
        action_json.extend(action_fixtures)
        af.seek(0)
        af.write(json.dumps(action_json, indent=4))
        print 'Ok'


def del_model_file(package, modul):
    target_file = os.path.join(get_app_location(package),
                               package, 'model', '%s.py' % modul)
    print 'Deleting model file "%s"... ' % target_file,
    try:
        os.remove(target_file)
        print 'Ok.'
    except Exception:
        print 'Failed.'


def add_model_file(package, modul, id, clazz, mixins):
    target_file = os.path.join(get_app_location(package),
                               package, 'model', '%s.py' % modul)
    print 'Adding new model file "%s"... ' % target_file,
    # Build mixins
    mixinclasses = []
    for mixin in mixins:
        mixinclasses.append(mixinmap[mixin])
    try:
        tablename = modul + 's'
        label = modul.capitalize()
        label_plural = label + "s"
        clazzpath = ".".join([package, 'model', modul, label])
        values = {
            'modul': modul,
            'clazzpath': clazzpath,
            'label': label,
            'label_plural': label_plural,
            'modul': modul,
            'table': tablename,
            'id': id,
            'clazz': clazz,
            'mixins': mixinclasses
        }
        template = template_lookup.get_template("model.mako")
        generated = template.render(**values)
        outfile = open(target_file, 'w+')
        outfile.write(generated)
        outfile.close()
        # Import the new model file. This is needed to be able to
        # generate the migration script properly.
        dynamic_import(clazzpath)
        print 'Ok.'
    except Exception:
        print 'Failed.'


def del_form_file(package, modul):
    filename = modul + "s"
    target_file = os.path.join(get_app_location(package),
                               package, 'views', 'forms', '%s.xml' % filename)
    print 'Deleting form configuration file "%s"... ' % target_file,
    try:
        os.remove(target_file)
        print 'Ok.'
    except:
        print 'Failed.'


def add_form_file(package, modul):
    filename = modul + "s"
    target_file = os.path.join(get_app_location(package),
                               package, 'views', 'forms', '%s.xml' % filename)
    print 'Adding new form configuration file "%s"... ' % target_file,
    try:
        values = {}
        template = template_lookup.get_template("form.mako")
        generated = template.render(**values)
        outfile = open(target_file, 'w+')
        outfile.write(generated)
        outfile.close()
        print 'Ok.'
    except:
        print 'Failed.'


def del_table_file(package, modul):
    filename = modul + "s"
    target_file = os.path.join(get_app_location(package),
                               package,
                               'views', 'tables', '%s.json' % filename)
    print 'Deleting table configuration file "%s"... ' % target_file,
    try:
        os.remove(target_file)
        print 'Ok.'
    except:
        print 'Failed.'


def add_table_file(package, modul):
    filename = modul + "s"
    target_file = os.path.join(get_app_location(package),
                               package,
                               'views', 'tables', '%s.json' % filename)
    print 'Adding new table configuration file "%s"... ' % target_file,
    try:
        values = {}
        template = template_lookup.get_template("table.json")
        generated = template.render(**values)
        outfile = open(target_file, 'w+')
        outfile.write(generated)
        outfile.close()
        print 'Ok.'
    except:
        print 'Failed.'


def handle_modul_fields_command(args):
    """Will print generated SA fields ready to be pasted in the model
    based on the given form configuration.

    :args: command args
    """
    path = []
    path.append(get_app_location(args.app))
    path.append(args.app)
    path.append("views")
    path.append("forms")
    path.append("%s.xml" % args.name)

    formbar_path = []
    formbar_path.append(get_app_location("formbar"))
    formbar_path.append("contrib")
    formbar_path.append("generate.py")

    print "\nGenerating fields for the %s model..." % args.name
    out = subprocess.check_output([
        "python",
        os.path.join(*formbar_path),
        "model",
        os.path.join(*path)
    ])
    print "\n", out


def handle_modul_delete_command(args):
    """@todo: Docstring for add_modul_add_command.

    :args: @todo
    :returns: @todo
    """
    path = []
    path.append(get_app_location(args.app))
    path.append(args.config)

    session = get_session(os.path.join(*path))
    package = args.app
    name = args.name
    remove_db_entry(name, session)
    del_model_file(package, name)
    del_form_file(package, name)
    del_table_file(package, name)


def handle_modul_add_command(args):
    """@todo: Docstring for add_modul_add_command.

    :args: @todo
    :returns: @todo
    """
    path = []
    path.append(get_app_location(args.app))
    path.append(args.config)

    session = get_session(os.path.join(*path))
    package = args.app
    name = args.name
    clazz = name.capitalize()
    modul_id = get_next_modulid(package, session)
    add_model_file(package, name, modul_id, clazz, args.mixin)
    add_form_file(package, name)
    add_table_file(package, name)
    msg = "Added %s modul" % name
    path = create_new_revision(args, msg)
    replace_sql_in_revision(path, package, name, session)
    print ""
    print "Ready. Next steps:"
    print ""
    print "touch %s" % path
    print "%s-admin db upgrade" % args.app



"""Removed not null constraint on uuid

Revision ID: 99fda072ff9
Revises: 1a553ec2b34b
Create Date: 2015-09-05 20:05:31.083712

"""

# revision identifiers, used by Alembic.
revision = '99fda072ff9'
down_revision = '1a553ec2b34b'

from alembic import op
import sqlalchemy as sa


UPGRADE = """"""
DOWNGRADE = """"""


def iter_statements(stmts):
    for st in [x for x in stmts.split('\n') if x]:
        op.execute(st)


def upgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.alter_column('actions', 'uuid',
               existing_type=sa.CHAR(length=32),
               nullable=True)
    op.alter_column('forms', 'uuid',
               existing_type=sa.CHAR(length=32),
               nullable=True)
    op.alter_column('modules', 'uuid',
               existing_type=sa.CHAR(length=32),
               nullable=True)
    op.alter_column('profiles', 'uuid',
               existing_type=sa.CHAR(length=32),
               nullable=True)
    op.alter_column('roles', 'uuid',
               existing_type=sa.CHAR(length=32),
               nullable=True)
    op.alter_column('usergroups', 'uuid',
               existing_type=sa.CHAR(length=32),
               nullable=True)
    op.alter_column('users', 'uuid',
               existing_type=sa.CHAR(length=32),
               nullable=True)
    ### end Alembic commands ###
    iter_statements(UPGRADE)


def downgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.alter_column('users', 'uuid',
               existing_type=sa.CHAR(length=32),
               nullable=False)
    op.alter_column('usergroups', 'uuid',
               existing_type=sa.CHAR(length=32),
               nullable=False)
    op.alter_column('roles', 'uuid',
               existing_type=sa.CHAR(length=32),
               nullable=False)
    op.alter_column('profiles', 'uuid',
               existing_type=sa.CHAR(length=32),
               nullable=False)
    op.alter_column('modules', 'uuid',
               existing_type=sa.CHAR(length=32),
               nullable=False)
    op.alter_column('forms', 'uuid',
               existing_type=sa.CHAR(length=32),
               nullable=False)
    op.alter_column('actions', 'uuid',
               existing_type=sa.CHAR(length=32),
               nullable=False)
    ### end Alembic commands ###
    iter_statements(DOWNGRADE)



# -*- coding: utf-8 -*-
import logging
import sqlalchemy as sa
import re
from pyramid.httpexceptions import (
    HTTPBadRequest,
    HTTPForbidden,
    HTTPFound,
    HTTPUnauthorized
)
from pyramid.security import forget
from pyramid.view import view_config
from pyramid.compat import urlparse
from formbar.form import Form, Validator

from ringo.views.base import create, rest_create, update
from ringo.views.helpers import get_item_from_request, get_current_form_page

from ringo.views.request import (
    handle_history,
    handle_params,
    handle_caching
)

from ringo.lib.form import get_form_config
from ringo.lib.helpers import import_model, get_action_routename
from ringo.lib.security import login, encrypt_password, has_permission
from ringo.lib.sql.cache import invalidate_cache

User = import_model('ringo.model.user.User')
Usergroup = import_model('ringo.model.user.Usergroup')
Role = import_model('ringo.model.user.Role')

log = logging.getLogger(__name__)


def user_create_callback(request, user):
    user = encrypt_password_callback(request, user)
    # Set profile data
    user.profile[0].first_name = request.params.get("_first_name")
    user.profile[0].last_name = request.params.get("_last_name")
    user.profile[0].email = request.params.get("_email")
    return user


def encrypt_password_callback(request, user):
    """Callback helper function. This function is called within the base
    create view to encrypt the password after the user has been
    created."""
    unencryped_pw = user.password
    user.password = encrypt_password(unencryped_pw)
    return user


def check_password(field, data):
    """Validator function as helper for formbar validators"""
    password = data[field]
    username = data["login"]
    return bool(login(username, password))


def user_update_callback(request, user):
    """Will rename the users usergroup in case the login has been
    changed. This is needed as the name of the usergroup is the only we
    to find out which usergroup belongs to the user."""
    if hasattr(request, '_oldlogin') and user.login != request._oldlogin:
        usergroup = request.db.query(Usergroup).filter(
            Usergroup.name == request._oldlogin).first()
        # Handle case if the user user not have his own usergroup. This
        # is true for the default admin user e.g
        if usergroup:
            usergroup.name = user.login
    return user

###########################################################################
#                               HTML VIEWS                                #
###########################################################################


def user_name_create_validator(field, data, db):
    """Validator to ensure username uniqueness when creating users"""
    return db.query(User).filter(User.login == data[field]).count() == 0


def user_name_update_validator(field, data, params):
    """Validator to ensure username uniqueness when updating users"""
    db = params['db']

    # Only consider names for the other users
    return db.query(User).filter(User.login == data[field],
                                 ~(User.id == params['pk'])).count() == 0


def password_nonletter_validator(field, data):
    """Validator to ensure that passwords contain two non-letters"""
    return len(re.findall('[^a-zA-ZäöüÄÖÜß]', data[field])) >= 2


def password_minlength_validator(field, data):
    """Validator to ensure that passwords are at least 12 characters long."""
    return len(data[field]) >= 12


@view_config(route_name=get_action_routename(User, 'create'),
             renderer='/default/create.mako',
             permission='create')
def create_(request, callback=None):
    """View to create new users. This view also take a optional callback
    parameter which can be used to inject additional callbacks in case
    this view is owerwritten in another application.

    :request: Current request
    :callback: Optional paramter for callback(s)
    """

    callbacks = []
    callbacks.append(user_create_callback)
    if callback:
        if isinstance(callback, list):
            callbacks.extend(callback)
        else:
            callbacks.append(callback)

    _ = request.translate
    uniqueness_validator = Validator('login',
                                     _('This name is already in use, '
                                       'please use something unique.'),
                                     user_name_create_validator,
                                     request.db)
    pw_len_validator = Validator('password',
                                 _('Password must be at least 12 characters '
                                   'long.'),
                                 password_minlength_validator)
    pw_nonchar_validator = Validator('password',
                                     _('Password must contain at least 2 '
                                       'non-letters.'),
                                     password_nonletter_validator)
    return create(request, callbacks,
                  validators=[uniqueness_validator,
                              pw_len_validator,
                              pw_nonchar_validator])


@view_config(route_name=get_action_routename(User, 'update'),
             renderer='/default/update.mako',
             permission='update')
def update_(request, callback=None, renderers=None,
           validators=None, values=None):
    user = get_item_from_request(request)
    # Store the login name of the user in the request to make it
    # available in the callback
    request._oldlogin = user.login

    _ = request.translate
    uniqueness_validator = Validator('login',
                                     _('This name is already in use, '
                                       'please use something unique.'),
                                     user_name_update_validator,
                                     {'pk': user.id, 'db': request.db})
    pw_len_validator = Validator('password',
                                 _('Password must be at least 12 characters '
                                   'long.'),
                                 password_minlength_validator)
    pw_nonchar_validator = Validator('password',
                                     _('Password must contain at least 2 '
                                       'non-letters.'),
                                     password_nonletter_validator)

    if validators is None:
        validators = []
    validators.append(uniqueness_validator)
    validators.append(pw_len_validator)
    validators.append(pw_nonchar_validator)

    callbacks = []
    callbacks.append(user_update_callback)
    if callback:
        if isinstance(callback, list):
            callbacks.extend(callback)
        else:
            callbacks.append(callback)

    return update(request, values=values,
                  validators=validators, callback=callbacks)


@view_config(route_name=get_action_routename(Usergroup, 'setstandin'),
             renderer='/usergroups/setstandin.mako')
def setstandin(request, allowed_users=None):
    """Setting members in the default usergroup of the current user.
    Technically this is adding a standin for this user."""

    # Check authentification
    # As this view has now security configured it is
    # generally callable by all users. For this reason we first check if
    # the user is authenticated. If the user is not authenticated the
    # raise an 401 (unauthorized) exception.
    if not request.user:
        raise HTTPUnauthorized

    # Check authorisation
    # For normal users users shall only be allowed to set the standin
    # for their own usergroup. So check this and otherwise raise an exception.
    usergroup = get_item_from_request(request)
    if (usergroup.id != request.user.default_gid
       and not has_permission("update", usergroup, request)):
        raise HTTPForbidden()

    clazz = Usergroup
    request.session['%s.form' % clazz] = "membersonly"
    request.session['%s.backurl' % clazz] = request.current_route_path()
    request.session.save()
    values = {}
    if allowed_users:
        values['_allowedusers'] = [u.login for u in allowed_users]

    # Result may be a HTTPFOUND object.
    result = update(request, values=values)
    if isinstance(result, dict):
        # If the standing is set by an administrational user then the id
        # of the usergroup´s user is stored in the the backurl.
        if request.GET.get('backurl'):
            user_id = urlparse.urlparse(
                request.GET.get('backurl')).path.split('/')[-1]
            user = request.db.query(User).filter(User.id == user_id).one()
        # Otherwise the user sets the standin of his own group. In this
        # case the user is already in the request.
        else:
            user = request.user
        result['user'] = user

    # Reset form value in session
    handle_caching(request)
    return result


@view_config(route_name=get_action_routename(User, 'changepassword'),
             renderer='/users/changepassword.mako')
def changepassword(request):
    """Method to change the users password by the user. The user user
    musst provide his old and the new pasword. Users are only allowed to
    change their own password."""

    # Check authentification
    # As this view has now security configured it is
    # generally callable by all users. For this reason we first check if
    # the user is authenticated. If the user is not authenticated the
    # raise an 401 (unauthorized) exception.
    if not request.user:
        raise HTTPUnauthorized

    clazz = User
    handle_history(request)
    handle_params(request)
    _ = request.translate
    rvalue = {}
    # Load the item return 400 if the item can not be found.
    id = request.matchdict.get('id')
    factory = clazz.get_item_factory()
    try:
        item = factory.load(id, request.db)
        # Check authorisation
        # User are only allowed to set their own password.
        if item.id != request.user.id:
            raise HTTPForbidden()
    except sa.orm.exc.NoResultFound:
        raise HTTPBadRequest()

    form = Form(get_form_config(item, 'changepassword'),
                item, request.db, translate=_,
                renderers={},
                change_page_callback={'url': 'set_current_form_page',
                                      'item': clazz.__tablename__,
                                      'itemid': id},
                request=request, csrf_token=request.session.get_csrf_token())

    if request.POST:
        mapping = {'item': item}
        # Do extra validation which is not handled by formbar.
        # Is the provided old password correct?
        validator = Validator('oldpassword',
                              _('The given password is not correct'),
                              check_password)
        pw_len_validator = Validator('password',
                                     _('Password must be at least 12 '
                                       'characters long.'),
                                     password_minlength_validator)
        pw_nonchar_validator = Validator('password',
                                         _('Password must contain at least 2 '
                                           'non-letters.'),
                                         password_nonletter_validator)

        form.add_validator(validator)
        form.add_validator(pw_len_validator)
        form.add_validator(pw_nonchar_validator)
        if form.validate(request.params):
            form.save()
            # Actually save the password. This is not done in the form
            # as the password needs to be encrypted.
            encrypt_password_callback(request, item)
            msg = _('Changed password for "${item}" successfull.',
                    mapping=mapping)
            log.info(msg)
            request.session.flash(msg, 'success')
            route_name = get_action_routename(item, 'changepassword')
            url = request.route_path(route_name, id=item.id)
            # Invalidate cache
            invalidate_cache()
            return HTTPFound(location=url)
        else:
            msg = _('Error on changing the password for '
                    '"${item}".', mapping=mapping)
            log.info(msg)
            request.session.flash(msg, 'error')

    rvalue['clazz'] = clazz
    rvalue['item'] = item
    rvalue['form'] = form.render(page=get_current_form_page(clazz, request))
    return rvalue


@view_config(route_name=get_action_routename(User, 'removeaccount'),
             renderer='/users/removeaccount.mako')
def removeaccount(request):
    """Method to remove the useraccout by the user."""

    # Check authentification
    # The view is only available for authenticated users and callable
    # if the user is not the admin unser (id=1)
    id = request.matchdict.get('id')
    if not request.user or id == '1':
        raise HTTPUnauthorized

    clazz = User
    _ = request.translate
    handle_history(request)
    handle_params(request)
    # Load the item return 400 if the item can not be found.
    factory = clazz.get_item_factory()

    try:
        item = factory.load(id, request.db)
        # Check authorisation
        if item.id != request.user.id:
            raise HTTPForbidden()
    except sa.orm.exc.NoResultFound:
        raise HTTPBadRequest()

    form = Form(get_form_config(item, 'removeaccount'),
                item, request.db, translate=_,
                renderers={},
                change_page_callback={'url': 'set_current_form_page',
                                      'item': clazz.__tablename__,
                                      'itemid': id},
                request=request, csrf_token=request.session.get_csrf_token())

    if request.POST:
        mapping = {'item': item}
        if form.validate(request.params):
            # Delete the account and redirect the user to a result page
            request.db.delete(item)
            headers = forget(request)
            target_url = request.route_path('users-accountremoved')
            return HTTPFound(location=target_url, headers=headers)
        else:
            msg = _('Deleting the account of '
                    '"${item}" failed.', mapping=mapping)
            log.info(msg)
            request.session.flash(msg, 'error')

    rvalue = {}
    rvalue['clazz'] = clazz
    rvalue['item'] = item
    rvalue['form'] = form.render(page=get_current_form_page(clazz, request))
    return rvalue

@view_config(route_name='users-accountremoved',
             renderer='/users/accountremoved.mako')
def accountremoved(request):
    return {}

def role_name_create_validator(field, data, db):
    """Validator to ensure username uniqueness when creating users"""
    return db.query(Role).filter(Role.name== data[field]).count() == 0


def role_name_update_validator(field, data, params):
    """Validator to ensure username uniqueness when updating users"""
    db = params['db']

    # Only consider names for the other users
    return db.query(Role).filter(Role.name == data[field],
                                 ~(Role.id == params['pk'])).count() == 0

@view_config(route_name=get_action_routename(Role, 'create'),
             renderer='/default/create.mako',
             permission='create')
def role_create_(request, callbacks=None):
    """View to create new roles.

    :request: Current request
    :callback: Optional paramter for callback(s)
    """

    _ = request.translate
    uniqueness_validator = Validator('name',
                                     _('This name is already in use, '
                                       'please use something unique.'),
                                     role_name_create_validator,
                                     request.db)
    return create(request, callbacks,
                  validators=[uniqueness_validator])


@view_config(route_name=get_action_routename(Role, 'update'),
             renderer='/default/update.mako',
             permission='update')
def role_update_(request, callback=None, renderers=None,
           validators=None, values=None):
    role = get_item_from_request(request)
    # Store the name of the role in the request to make it
    # available in the callback
    request._oldname = role.name

    _ = request.translate
    uniqueness_validator = Validator('name',
                                     _('This name is already in use, '
                                       'please use something unique.'),
                                     role_name_update_validator,
                                     {'pk': role.id, 'db': request.db})
    if validators is None:
        validators = []
    validators.append(uniqueness_validator)

    callbacks = []
    if callback:
        if isinstance(callback, list):
            callbacks.extend(callback)
        else:
            callbacks.append(callback)

    return update(request, values=values,
                  validators=validators, callback=callbacks)


###########################################################################
#                               REST SERVICE                              #
###########################################################################


@view_config(route_name=get_action_routename(User, 'create', prefix="rest"),
             renderer='json',
             request_method="POST",
             permission='create')
def rest_create_(request):
    return rest_create(User, request, user_create_callback)



#!/usr/bin/python
# Copyright (C) 2014 Torsten Irlaender <torsten@irlaender.de>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston,
# MA 02111-1307, USA.

"""
This is a short description of what this script does.
"""

import os
import sys
import argparse

from ringo.scripts.db import (
    handle_db_init_command,
    handle_db_revision_command,
    handle_db_upgrade_command,
    handle_db_downgrade_command,
    handle_db_savedata_command,
    handle_db_loaddata_command,
    handle_db_uuid_command,
    handle_db_restrict_command,
    handle_db_unrestrict_command,
    handle_db_fixsequence_command
)

from ringo.scripts.fixture import (
    handle_fixture_load_command,
    handle_fixture_save_command
)

from ringo.scripts.modul import (
    handle_modul_add_command,
    handle_modul_delete_command,
    handle_modul_fields_command,
)

from ringo.scripts.user import (
    handle_user_passwd_command
)

from ringo.scripts.application import (
    handle_app_init_command,
    handle_ext_init_command,
    handle_ext_delete_command
)

def get_config_path(config="development.ini"):
    return os.path.join(os.getcwd(), config)

def modul_name(var):
    return str(var).lower()

def setup_fixture_parser(subparsers, parent):
    p = subparsers.add_parser('fixtures',
                              help='Fixture loading and saving',
                              parents=[parent])
    sp = p.add_subparsers(help='Fixture command help')

    # Load
    fixture_parser = sp.add_parser('load',
                                help=('Loads all fixture files.'),
                                parents=[parent])
    fixture_parser.set_defaults(func=handle_fixture_load_command)
    fixture_parser.add_argument('--path',
                              metavar='path',
                              help='Path to the fixture files')

    # Save
    fixture_parser = sp.add_parser('save',
                                help=('Saves all fixture files.'),
                                parents=[parent])
    fixture_parser.set_defaults(func=handle_fixture_save_command)
    fixture_parser.add_argument('--path',
                              metavar='path',
                              help='Path to the fixture files')
    return sp


def setup_application_parser(subparsers, parent):
    p = subparsers.add_parser('app',
                              help='Application administration',
                              parents=[parent])
    sp = p.add_subparsers(help='Application command help')

    # Init command
    app_parser = sp.add_parser('init',
                                help=('Initialises an empty application '
                                      'folder and creates a default '
                                      'configuration file in it'),
                                parents=[parent])
    app_parser.add_argument('name',
                              metavar='name',
                              help='Name of the application')
    app_parser.set_defaults(func=handle_app_init_command)

    reg_parser = sp.add_parser('add-extension',
                                help=('Will create modul entries for a given extension'),
                                parents=[parent])
    reg_parser.add_argument('name',
                            metavar='name',
                            help='Name of the extension')
    reg_parser.set_defaults(func=handle_ext_init_command)
    unreg_parser = sp.add_parser('remove-extension',
                                help=('Will delete modul entries for a given extension'),
                                parents=[parent])
    unreg_parser.add_argument('name',
                            metavar='name',
                            help='Name of the extension')
    unreg_parser.set_defaults(func=handle_ext_delete_command)
    return sp


def setup_user_parser(subparsers, parent):
    p = subparsers.add_parser('user',
                              help='User administration',
                              parents=[parent])
    sp = p.add_subparsers(help='User command help')

    # Init command
    passwd_parser = sp.add_parser('passwd',
                                help=('Set the password for a user.'
                                      'Password will be autogenerated unless'
                                      'your provide the password with the '
                                      '--password option.'),
                                parents=[parent])
    passwd_parser.add_argument('user',
                              metavar='user',
                              help='Name of the user')
    passwd_parser.add_argument('--password',
                              metavar='password',
                              help='Password for the user')
    passwd_parser.set_defaults(func=handle_user_passwd_command)
    return sp


def setup_modul_parser(subparsers, parent):
    p = subparsers.add_parser('modul',
                              help='Modul administration',
                              parents=[parent])
    sp = p.add_subparsers(help='Modul command help')
    # Model command
    model_parser = sp.add_parser('fields',
                                 help='Generates the model fields',
                                 parents=[parent])
    model_parser.add_argument('name',
                              type=modul_name,
                              help='Name of the new modul (singular form)')
    model_parser.set_defaults(func=handle_modul_fields_command)

    # Add command
    add_parser = sp.add_parser('add',
                                help='Adds a new modul',
                                parents=[parent])
    add_parser.add_argument('name',
                            type=modul_name,
                            help='Name of the new modul (singular form)')
    add_parser.add_argument('--mixin',
                            nargs="*",
                            choices=['owned', 'meta', 'logged', 'state',
                                     'blobform', 'versioned', 'printable',
                                     'nested', 'commented', 'tagged', 'todo'],
                            default=["owned"],
                            help='Mixins for the generated model')
    add_parser.set_defaults(func=handle_modul_add_command)

    # Delete command
    delete_parser = sp.add_parser('delete',
                                  help='Deletes a modul',
                                  parents=[parent])
    delete_parser.add_argument('name',
                               type=modul_name,
                               help='Name of the modul to delete')
    delete_parser.set_defaults(func=handle_modul_delete_command)
    return sp


def setup_db_parser(subparsers, parent):
    p = subparsers.add_parser('db',
                              help='Database administration',
                              parents=[parent])
    sp = p.add_subparsers(help='Database command help')

    # Init command
    init_parser = sp.add_parser('init',
                                help='Initialise a new database',
                                parents=[parent])
    init_parser.set_defaults(func=handle_db_init_command)

    # Upgrade command
    upgrade_parser = sp.add_parser('upgrade',
                                help='Upgrades a database',
                                parents=[parent])
    upgrade_parser.set_defaults(func=handle_db_upgrade_command)

    # Downgrade command
    downgrade_parser = sp.add_parser('downgrade',
                                help='Downgrades a database',
                                parents=[parent])
    downgrade_parser.set_defaults(func=handle_db_downgrade_command)

    # Revision command
    revision_parser = sp.add_parser('revision',
                                help='Creates a new alembic revision command',
                                parents=[parent])
    revision_parser.set_defaults(func=handle_db_revision_command)

    # Savedata command
    savedata_parser = sp.add_parser('savedata',
                                help='Saves the data of a given modul',
                                parents=[parent])
    savedata_parser.set_defaults(func=handle_db_savedata_command)
    savedata_parser.add_argument('modul',
                        metavar="modul",
                        help="Name of the Modul")
    savedata_parser.add_argument('--include-relations',
                        action="store_true",
                        help="Include relations in the export")
    savedata_parser.add_argument('--format',
                        choices=["json", "csv"],
                        default="json",
                        help="Format of the saved data")

    # restrict command
    savedata_parser = sp.add_parser('restrict',
                                help=('Restrict access to the given modul'
                                     ' on database level'),
                                parents=[parent])
    savedata_parser.set_defaults(func=handle_db_restrict_command)
    savedata_parser.add_argument('modul',
                        metavar="modul",
                        help="Name of the Modul")
    savedata_parser = sp.add_parser('unrestrict',
                                help=('Unrestrict access to the given modul'
                                     ' on database level'),
                                parents=[parent])
    savedata_parser.set_defaults(func=handle_db_unrestrict_command)
    savedata_parser.add_argument('modul',
                        metavar="modul",
                        help="Name of the Modul")

    # Loaddata command
    loaddata_parser = sp.add_parser('loaddata',
                                help='Loads the data of a given modul',
                                parents=[parent])
    loaddata_parser.set_defaults(func=handle_db_loaddata_command)
    loaddata_parser.add_argument('modul',
                        metavar="modul",
                        help="Name of the Modul")
    loaddata_parser.add_argument('fixture',
                        metavar="fixture",
                        help="Path to the importfile")
    loaddata_parser.add_argument('--loadbyid',
                        action="store_true",
                        help="Load data by id and not by uuid")
    loaddata_parser.add_argument('--format',
                        choices=["json", "csv"],
                        default="json",
                        help="Format of the loaded data")

    # UUID command
    uuid_parser = sp.add_parser('resetuuid',
                                help='Generates UUID for data of a given modul',
                                parents=[parent])
    uuid_parser.set_defaults(func=handle_db_uuid_command)
    uuid_parser.add_argument('modul',
                        metavar="modul",
                        help="Name of the Modul")
    uuid_parser.add_argument('--missing-only',
                        action="store_true",
                        help="Reset the UUID only where it is not already set.")

    # Fix sequence command
    upgrade_parser = sp.add_parser('fixsequence',
                                help='Fixes sequences in postgres databases',
                                parents=[parent])
    upgrade_parser.set_defaults(func=handle_db_fixsequence_command)
    return sp


def setup_global_argument_parser():
    parser = argparse.ArgumentParser(add_help=False)
    parser.add_argument('--app',
                        default=_get_app_name(),
                        metavar="APP",
                        help="Name of the application")
    parser.add_argument('--base',
                        default="ringo",
                        metavar="BASEAPP",
                        help="Name of the base application")
    parser.add_argument('--config',
                        default=get_config_path(),
                        metavar="INI",
                        help="Configuration file for the application")
    return parser


def setup_parser():
    parser = {}
    parser["root"] = argparse.ArgumentParser(description="Administrate "
                                             "various aspects in a ringo "
                                             "based application.")
    global_arguments = setup_global_argument_parser()
    subparsers = parser["root"].add_subparsers(help='Command help')

    parser["db"] = setup_db_parser(subparsers, global_arguments)
    parser["modul"] = setup_modul_parser(subparsers, global_arguments)
    parser["user"] = setup_user_parser(subparsers, global_arguments)
    parser["app"] = setup_application_parser(subparsers, global_arguments)
    parser["fixture"] = setup_fixture_parser(subparsers, global_arguments)
    return (parser, subparsers, global_arguments)


def _get_app_name():
    """@todo: Docstring for _get_app_name.
    :returns: @todo

    """
    return sys.argv[0].split("/")[-1].split("-")[0]


def main():
    parser, subparsers, global_arguments = setup_parser()
    args = parser["root"].parse_args()
    args.func(args)

if __name__ == '__main__':
    main()

# vim: ai ts=4 sts=4 et sw=4



"""Modul for the messanging system in ringo"""
import logging
import os
import pkg_resources
from pyramid_mailer import get_mailer
from pyramid_mailer.message import Message

from mako.lookup import TemplateLookup
from ringo.lib.helpers.appinfo import get_app_name, get_app_title

log = logging.getLogger(__name__)


class Mailer:
    def __init__(self, request):
        self.mailer = get_mailer(request)
        self.default_sender = request.registry.settings['mail.default_sender']
        settings = request.registry.settings
        if (bool(settings.get('mail.host')) and
           bool(settings.get('mail.default_sender'))):
            self.enabled = True
        else:
            self.enabled = False

    def send(self, mail):
        """Will send and email with the subject and body to the recipient
        from the sender

        :recipient: The recipients mail address
        :sender: The senders mail address
        :subject: String of subject
        :message: Mails body.
        :returns: True or False

        """
        log.debug("Sending from '%s' to '%s'. Subject: %s" % (mail.sender or self.default_sender, mail.recipients, mail.subject))
        message = Message(subject=mail.subject,
                          sender=mail.sender or self.default_sender,
                          recipients=mail.recipients,
                          body=mail.body)
        if self.enabled:
            self.mailer.send(message)
        else:
            log.warning("Sending mail aborted. Mail system not configured")


class Mail:

    def __init__(self, recipients, subject,
                 template=None, values={}, msg="", sender=None):
        self.recipients = recipients
        self.subject = u"%s" % subject
        self.sender = sender
        self.body = ""

        app_base_dir = pkg_resources.get_distribution(get_app_name()).location
        ringo_base_dir = pkg_resources.get_distribution("ringo").location
        template_dir = os.path.join(app_base_dir,
                                    get_app_name(),
                                    'templates/mails')
        ringo_template_dir = os.path.join(ringo_base_dir,
                                          "ringo",
                                          'templates/mails')
        self.tl = TemplateLookup(directories=[template_dir,
                                              ringo_template_dir],
                                 default_filters=['h'])

        if template:
            self.template = self.tl.get_template("%s.mako" % template)
            self.body = self.template.render(**values)
        elif msg:
            self.body = msg
        else:
            raise Exception("Mail is missing either a"
                            " template and values or a msg")
        log.debug(self.body)



import sqlalchemy as sa
from ringo.model import Base
from ringo.model.base import BaseItem
from ringo.model.statemachine import (
    Statemachine, State,
    null_handler as handler,
    null_condition as condition
)
from ringo.model.mixins import Owned, Meta, StateMixin

d1 = """The form is currently marked as draft. This means it is
currently under work and may change."""
d2 = """The form  is published and can be used in the application. The
form can not be changed."""


class ReviewStatemachine(Statemachine):

    def setup(self):
        s1 = State(self, 1, "Draft", description=d1)
        s2 = State(self, 2, "Published", description=d2)

        s1.add_transition(s2, "Publish", handler, condition)
        s2.add_transition(s1, "Revise", handler, condition)
        return s1


class ReviewStateMixin(StateMixin):
    """Mixin to add Review state functionallity to the Forms"""
    _statemachines = {'review_state_id': ReviewStatemachine}
    review_state_id = sa.Column(sa.Integer, default=1)

    @property
    def review_state(self):
        state = self.get_statemachine('review_state_id')
        return state.get_state()


class Form(BaseItem, ReviewStateMixin, Owned, Meta, Base):
    __tablename__ = 'forms'
    _modul_id = 14
    id = sa.Column(sa.Integer, primary_key=True)
    category = sa.Column(sa.Integer)
    title = sa.Column(sa.String, nullable=True, default='')
    description = sa.Column(sa.Text, nullable=True, default='')
    definition = sa.Column(sa.Text, nullable=True, default='')
    mid = sa.Column(sa.Integer, sa.ForeignKey('modules.id'))

    # relations
    modul = sa.orm.relationship("ModulItem", backref="blobforms")



def parse_setting(name, settings, default=None):
    value = settings.get(name) if settings.get(name, None) else default
    if value and not isinstance(value, list):
        values = []
        for v in value.split(" "):
            if v in ['self', 'unsafe-inline', 'none', 'unsafe-eval']:
                values.append("'%s'" % v)
            else:
                values.append("%s" % v)
        return values
    else:
        return value


def build_csp_policy(settings):
    # TODO: Remove "unsafe-inline" default here. Change Ringo in the way
    # that it does not contain inline javascript! (ti) <2015-06-03 17:10>
    config = {
        'default-src': parse_setting('security.csp.default_src', settings,
                                     ["'self'"]),
        'script-src': parse_setting('security.csp.script_src',
                                    settings,
                                    ["'self'", "'unsafe-inline'"]),
        'object-src': parse_setting('security.csp.object_src', settings),
        'style-src': parse_setting('security.csp.style_src',
                                   settings,
                                   ["'self'", "'unsafe-inline'"]),
        'img-src': parse_setting('security.csp.img_src', settings),
        'media-src': parse_setting('security.csp.media_src', settings),
        'frame-src': parse_setting('security.csp.frame_src', settings),
        'frame-ancestors': parse_setting('security.csp.frame_ancestors',
                                         settings,
                                         ["'none'"]),
        'font-src': parse_setting('security.csp.font_src', settings),
        'connect-src': parse_setting('security.csp.connect_src', settings),
        'sandbox': parse_setting('security.csp.sandbox', settings),
    }
    policy = ['{0} {1}'.format(k, ' '.join(v)) for (k, v) in
              config.iteritems() if v]
    if settings.get('csp.report_uri', None):
        policy.append('report-uri {0}'.format(settings.get('csp.report_uri')))
    return '; '.join(policy)


def csp_factory(handler, registry):
    def csp_tween(request):
        settings, response = registry.settings, request.response
        HEADERS = ['Content-Security-Policy', 'X-WebKit-CSP']

        prefixes = settings.get('csp.exclude_prefixes', ('/admin'))
        if request.path.startswith(prefixes):
            return handler(request)
        if response.headers.get(HEADERS[0], None) is None:
            policy = build_csp_policy(settings)
            for HEADER in HEADERS:
                response.headers[HEADER] = policy
        return handler(request)
    return csp_tween



import logging
from ringo.views.response import JSONResponse
from ringo.views.helpers import (
    get_item_form,
    render_item_form
)
from ringo.views.request import (
    handle_params,
    handle_history,
    handle_POST_request,
    handle_redirect_on_success,
    get_item_from_request,
    get_return_value
)

log = logging.getLogger(__name__)


def update(request, callback=None, renderers=None,
           validators=None, values=None):
    """Base method to handle update requests. This view will render a
    update form to update items on (GET) requests.

    It the user submits the data (POST) that the data will be validated.
    If validation is successfull the item will be saved in the datebase
    and the callback method will be called. Finally a redirect to either
    the backurl or the edit or read mode of the item will be triggered.

    If the validation fails, the form will be rendered again.
    :request: Current request
    :callback: Current function which is called after the item has been read.
    :renderers: Dictionary of external renderers which should be used
                for renderering some form elements.
    :validators: List of external formbar validators which should be
                 added to the form for validation
    :values: Dictionary of additional values which will be available in
             the form
    :returns: Dictionary or Redirect.
    """
    handle_history(request)
    handle_params(request)
    if values is None:
        values = {}
    values['_roles'] = [str(r.name) for r in request.user.roles]
    form = get_item_form('update', request, renderers, validators, values)
    if request.POST:
        if handle_POST_request(form, request, callback, 'update', renderers):
            return handle_redirect_on_success(request)

    rvalues = get_return_value(request)
    rvalues['form'] = render_item_form(request, form, values)
    return rvalues


def rest_update(request, callback=None):
    """Updates an item of type clazz. The item is loaded based on the
    unique id value provided in the matchtict object in the current
    request. The item will be updated with the data submitted in the
    current PUT request. Before updating the item the data will be
    validated against the "update" form of the item. If the validation
    fails the item will not be updated. In all cases the item is return as
    JSON object with the item and updated values back to the client. The
    JSON Response will include further details on the reason why the
    validation failed.

    :request: Current request
    :returns: JSON object.
    """
    item = get_item_from_request(request)
    form = get_item_form('update', request)
    if form.validate(request.params):
        item.save(form.data, request)
        return JSONResponse(True, item)
    else:
        # Validation fails! return item
        return JSONResponse(False, item)



import logging
import os
import cgi
import pkg_resources
from mako.lookup import TemplateLookup

from formbar.config import Config, load
from formbar.form import Form

import ringo.lib.helpers
from ringo.lib.helpers import (
    get_action_routename,
    get_item_modul,
    get_app_url,
    literal,
    escape,
    HTML
)
from ringo.lib.form import (
    get_eval_url,
    get_path_to_form_config,
)

base_dir = pkg_resources.get_distribution("ringo").location
template_dir = os.path.join(base_dir, 'ringo', 'templates')
template_lookup = TemplateLookup(directories=[template_dir],
                                 default_filters=['h'])

log = logging.getLogger(__name__)


###########################################################################
#                         Renderers for dialogs                           #
###########################################################################


class DialogRenderer(object):
    """Renderer for Dialogs"""

    def __init__(self, request, item, action, title=None, body=None):
        """Renders a renderered dialog for the requested action on the
        given item. If not header or body is provided the dialog will
        have a default message.

        :request: The current request
        :item: The item for which the item should be confirmed.
        :action: The actions which must be confirmend.
        :header: Custom text for the header of the dialog
        :body: Custom text for the body of the dialog.

        """
        self._request = request
        self._item = item
        self._action = action
        self._title = title
        self._body = body

    def render(self):
        """Initialize renderer"""
        pass


class ConfirmDialogRenderer(DialogRenderer):
    """Docstring for ConfirmDialogRenderer """

    def __init__(self, request, clazz, action,
                 title=None, body=None, ok_label=None):
        """@todo: to be defined """
        DialogRenderer.__init__(self, request, clazz, action, title, body)
        self.ok_label = ok_label
        self.template = template_lookup.get_template("internal/confirm.mako")
        self.icon = self._request.static_path(
            'ringo:static/images/icons/32x32/dialog-warning.png')

    def render(self, items):
        _ = self._request.translate
        mapping = {'Action': _(self._action.capitalize())}
        values = {}
        values['request'] = self._request
        values['icon'] = self.icon
        if self._title:
            values['header'] = self._title
        else:
            values['header'] = _("Confirm ${Action}", mapping=mapping)
        if self._body:
            values['body'] = self._body
        else:
            values['body'] = self._render_body(items)
        values['action'] = self.ok_label or _(self._action.capitalize())
        values['ok_url'] = self._request.current_route_path()
        values['_'] = self._request.translate
        values['cancel_url'] = self._request.referrer
        values['eval_url'] = self._request.application_url+get_eval_url()
        return literal(self.template.render(**values))

    def _render_body(self, items):
        out = []
        _ = self._request.translate
        item_label = escape(get_item_modul(self._request,
                                           self._item).get_label())
        mapping = {'action': escape(_(self._action.capitalize()).lower()),
                   'item': item_label,
                   'Action': escape(_(self._action.capitalize()))}
        out.append(_("Do you really want to ${action}"
                     " the following ${item} items?",
                     mapping=mapping))
        out.append(HTML.tag("br", _closed=False))
        out.append(HTML.tag("ol", _closed=False))
        for item in items:
            out.append(HTML.tag("li", _closed=False))
            out.append(escape(unicode(item)))
            out.append(HTML.tag("/li", _closed=False))
        out.append(HTML.tag("/ol", _closed=False))
        out.append(_('Please press "${Action}" to ${action} the item.'
                     ' Press "Cancel" to cancel the action.',
                     mapping=mapping))

        return literal("").join(out)


class ErrorDialogRenderer(DialogRenderer):
    """Docstring for ErrorDialogRenderer """

    def __init__(self, request, title, body):
        """@todo: to be defined """
        DialogRenderer.__init__(self, request, None, None, title, body)
        self.template = template_lookup.get_template("internal/error.mako")
        self.icon = self._request.static_path(
            'ringo:static/images/icons/32x32/dialog-error.png')

    def render(self, url=None):
        values = {}
        values['icon'] = self.icon
        values['header'] = self._title
        values['body'] = self._body
        history = self._request.session.get('history')
        if url:
            values['ok_url'] = url
        elif history:
            values['ok_url'] = self._request.session['history'].pop()
        else:
            values['ok_url'] = self._request.route_path('home')
        values['eval_url'] = self._request.application_url+get_eval_url()
        return literal(self.template.render(**values))


class WarningDialogRenderer(ErrorDialogRenderer):
    def __init__(self, request, title, body):
        """@todo: to be defined """
        ErrorDialogRenderer.__init__(self, request, title, body)
        self.template = template_lookup.get_template("internal/warning.mako")
        self.icon = self._request.static_path(
            'ringo:static/images/icons/32x32/dialog-warning.png')


class InfoDialogRenderer(ErrorDialogRenderer):
    def __init__(self, request, title, body):
        """@todo: to be defined """
        ErrorDialogRenderer.__init__(self, request, title, body)
        self.template = template_lookup.get_template("internal/info.mako")
        self.icon = self._request.static_path(
            'ringo:static/images/icons/32x32/dialog-information.png')


class ExportDialogRenderer(DialogRenderer):
    """Docstring for ExportDialogRenderer"""

    def __init__(self, request, clazz):
        """@todo: to be defined """
        DialogRenderer.__init__(self, request, clazz, "export")
        self.template = template_lookup.get_template("internal/export.mako")
        config = Config(load(get_path_to_form_config('export.xml', 'ringo')))
        form_config = config.get_form('default')
        self.form = Form(form_config,
                         csrf_token=self._request.session.get_csrf_token(),
                         dbsession=request.db,
                         eval_url=get_eval_url(),
                         url_prefix=get_app_url(request))

    def render(self, items):
        _ = self._request.translate
        values = {}
        values['request'] = self._request
        values['items'] = items
        values['body'] = self._render_body()
        values['modul'] = get_item_modul(self._request, self._item).get_label(plural=True)
        values['action'] = _(self._action.capitalize())
        values['ok_url'] = self._request.current_route_path()
        values['_'] = self._request.translate
        values['cancel_url'] = self._request.referrer
        values['eval_url'] = self._request.application_url+get_eval_url()
        return literal(self.template.render(**values))

    def _render_body(self):
        # The output of the form render function of formbar is
        # considered safe here as formbar already handles the escapeing.
        # So we put it into the literal function to mark it save.
        return literal(self.form.render(buttons=False))


class ImportDialogRenderer(DialogRenderer):
    """Docstring for ImportDialogRenderer"""

    def __init__(self, request, clazz):
        """@todo: to be defined """
        DialogRenderer.__init__(self, request, clazz, "import")
        self.template = template_lookup.get_template("internal/import.mako")
        config = Config(load(get_path_to_form_config('import.xml', 'ringo')))
        form_config = config.get_form('default')
        self.form = Form(form_config,
                         csrf_token=self._request.session.get_csrf_token(),
                         dbsession=request.db,
                         eval_url=get_eval_url(),
                         url_prefix=get_app_url(request))

    def render(self, items):
        values = {}
        values['request'] = self._request
        values['body'] = self._render_body()
        values['modul'] = get_item_modul(self._request, self._item).get_label(plural=True)
        values['action'] = self._action.capitalize()
        values['ok_url'] = self._request.current_route_path()
        values['_'] = self._request.translate
        values['cancel_url'] = self._request.referrer
        values['overview_url'] = self._request.route_path(get_action_routename(self._item, 'list'))
        values['eval_url'] = self._request.application_url+get_eval_url()
        values['items'] = items
        values['h'] = ringo.lib.helpers
        return literal(self.template.render(**values))

    def _render_body(self):
        # The output of the form render function of formbar is
        # considered safe here as formbar already handles the escapeing.
        # So we put it into the literal function to mark it save.
        return literal(self.form.render(buttons=False))



"""Added Owner and Group to roles, usergroups, users

Revision ID: 1a553ec2b34b
Revises: 3520cf26d8d8
Create Date: 2015-07-21 23:31:22.487231

"""

# revision identifiers, used by Alembic.
revision = '1a553ec2b34b'
down_revision = '3520cf26d8d8'

from alembic import op
import sqlalchemy as sa

UPGRADE = """"""
DOWNGRADE = """"""


def iter_statements(stmts):
    for st in [x for x in stmts.split('\n') if x]:
        op.execute(st)


def upgrade():
    ### commands auto generated by Alembic - please adjust! ###
    with op.batch_alter_table("roles") as batch_op:
        batch_op.add_column(sa.Column('gid', sa.Integer(), nullable=True))
        batch_op.add_column(sa.Column('uid', sa.Integer(), nullable=True))
        batch_op.create_foreign_key('fk_roles_gid_usergroups', 'usergroups', ['gid'], ['id'])
        batch_op.create_foreign_key('fk_roles_uid_usergroups', 'users', ['uid'], ['id'])

    with op.batch_alter_table("usergroups") as batch_op:
        batch_op.add_column(sa.Column('gid', sa.Integer(), nullable=True))
        batch_op.add_column(sa.Column('uid', sa.Integer(), nullable=True))
        batch_op.create_foreign_key('fk_usergroups_gid_usergroups', 'usergroups', ['gid'], ['id'])
        batch_op.create_foreign_key('fk_usergroups_uid_users', 'users', ['uid'], ['id'])
    with op.batch_alter_table("users") as batch_op:
        batch_op.add_column(sa.Column('gid', sa.Integer(), nullable=True))
        batch_op.add_column(sa.Column('uid', sa.Integer(), nullable=True))
        #  FIXME: Migration fails on SQLite when the following FK
        #  are enabled. As alembic in combination with sqlite does have
        #  many issues anyway we will disable the FK contrains for
        #  Sqlite (ti) <2015-07-22 09:17> 
        if batch_op.migration_context.dialect.name != 'sqlite':
            batch_op.create_foreign_key('fk_users_gid_usergroups', 'usergroups', ['gid'], ['id'])
            batch_op.create_foreign_key('fk_users_uid_users', 'users', ['uid'], ['id'])
    ### end Alembic commands ###
    iter_statements(UPGRADE)


def downgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.drop_column('users', 'uid')
    op.drop_column('users', 'gid')
    op.drop_column('usergroups', 'uid')
    op.drop_column('usergroups', 'gid')
    op.drop_column('roles', 'uid')
    op.drop_column('roles', 'gid')
    ### end Alembic commands ###
    iter_statements(DOWNGRADE)



from invoke import run
from ringo.scripts.helpers import get_db, get_package_name, get_fixtures


def handle_fixture_load_command(args):
    appname = get_package_name(args.config)
    db = get_db(args.config)
    print "Loading data in %s" % db
    for fixture, modul in get_fixtures(args.app, args.path):
        print "Loading fixture %s" % fixture
        run("%s-admin db loaddata --loadbyid --config %s %s %s" % (appname, args.config, modul, fixture))


def handle_fixture_save_command(args):
    appname = get_package_name(args.config)
    db = get_db(args.config)
    print "Saving data in %s" % db
    for fixture, modul in get_fixtures(args.app, args.path):
        print "Saving fixture %s" % fixture
        try:
            run("%s-admin db savedata --include-relations --config %s %s > %s" % (appname, args.config, modul, fixture), hide="stderr")
        except:
            print "Ups... Trying again without relations included"
            run("%s-admin db savedata --config %s %s > %s" % (appname, args.config, modul, fixture))
        run("python -m json.tool %s > %s.tmp" % (fixture, fixture))
        run("mv %s.tmp %s" % (fixture, fixture))



import logging
import string
from datetime import datetime
from pyramid.threadlocal import get_current_request
from ringo.lib.sql import DBSession

log = logging.getLogger(__name__)


def serialize(value):
    """Very simple helper function which returns a stringified version
    of the given python value."""
    if value is None:
        return ""
    if isinstance(value, unicode):
        return value
    if isinstance(value, int):
        return unicode(value)
    if isinstance(value, float):
        return unicode(value)
    if isinstance(value, datetime):
        return value.strftime("%Y-%m-%d %H:%M:%S")
    if isinstance(value, bool):
        # use 1 and 0 here to be able to set this values on import,
        # "True" and "False" will fail.
        return value and "1" or "0"
    # Now handle relations, 3 cases: empty, 1 item and 2+ items.
    if isinstance(value, list) and not value:
        return []
    # cannot import BaseItem here, so check for BaseItem with hasattr.
    if isinstance(value, list) and hasattr(value[0], 'id'):
        return sorted([v.id for v in value])
    if hasattr(value, 'id'):
        return [value.id]
    # Even if Ringo does not have a bytearray type yet the serialize
    # method supports it to convert the given value into unicode
    if isinstance(value, bytearray):
        return value.decode("utf-8")
    log.warning("Unhandled type '%s'. "
                "Using default and converting to unicode" % type(value))
    return unicode(value)

def safestring(unsafe):
    """Returns a 'safe' version of the given string. All non ascii chars
    and other chars are removed """
    valid_chars = "_%s%s" % (string.ascii_letters, string.digits)
    return ''.join(c for c in unsafe if c in valid_chars)


def age(when, on=None):
    """Returns the age in years from the given date "when" since today. If
    "on" is provided the age will be calculated relative to the given
    date. See http://keyes.ie/calculate-age-with-python/

    :when: datetime object
    :on: datetime object
    :returns: integer

    """
    if on is None:
        on = datetime.today()
    was_earlier = (on.month, on.day) < (when.month, when.day)
    return on.year - when.year - (was_earlier)


def _resolve_attribute(element, name, idx=None):
    """Helper method for the set_raw_value and get_raw_value method.
    Will return on default the last element in a dot
    separated attribute. E.g. for a attribute called 'foo.bar.baz' the
    function will return a the 'baz' element.

    The default element to return can can be changed by providing the
    idx attribute. The idx will be used to iterate over the splitted
    attribute name. ([0:idx]). This is usually only used when trying to
    set a value. See set_raw_value for more details.

    The function also offers basic support for accessing individual
    elements in lists in case one attribute is a list while iterating
    over all attributes. Currently only flat lists are supported.

    Example: x.y[1].z

    The method is tolerant agains errors when trying to resolve elements
    in lists which are not present. This is likly the case for relations
    when x has no elements stored in relation y. In this case the method
    stops resolving the value and returns None. However we log this
    situation in the debugging output as it might be a sign for an
    error.

    :element: BaseItem instance
    :name: dot separated attribute name
    :returns: last element of the dot separated element.

    """
    attributes = name.split('.')
    if idx is None:
        idx = len(attributes)
    for attr in attributes[0:idx]:
        splitmark_s = attr.find("[")
        splitmark_e = attr.find("]")
        if splitmark_s > 0:
            index = int(attr[splitmark_s + 1:splitmark_e])
            attr = attr[:splitmark_s]
            element_list = object.__getattribute__(element, attr)
            if len(element_list) > 0:
                element = element_list[index]
            else:
                log.debug("IndexError in %s on %s for %s"
                          % (name, attr, element))
                element = None
                break
        else:
            element = object.__getattribute__(element, attr)
    return element


def set_raw_value(element, name, value):
    """Support setting the `value` for a dot separated attribute of
    `element` provided by the `name` attribute. If the attribute name is
    a dot separated attribute the function will first resolve the
    attribute to the penultimate element and call the setattr method for
    the ultimate element and the given value.

    E.g. If the value '123' should be set for the attribute
    'foo.bar.baz' the function first gets the 'bar' element.  For this
    element the setattr method will be called with the attribute 'baz'
    and the value '123'."""

    if name.find(".") > -1:
        penulti = _resolve_attribute(element, name, -1)
        attr = name.split(".")[-1]
    else:
        penulti = element
        attr = name
    object.__setattr__(penulti, attr, value)


def get_raw_value(element, name):
    """Support getting the value for a dot separated attribute of
    `element` provided by the `name` attribute. If the attribute name is
    a dot separated attribute the function will first resolve the
    attribute to the penultimate element and call the getattr method for
    the ultimate element and the given value.

    E.g. If the function should get the attribute 'foo.bar.baz' the
    function first gets the 'bar' element.  For this element the getattr
    method will be called with the attribute 'baz'."""

    return _resolve_attribute(element, name)


def dynamic_import(cl):
    d = cl.rfind(".")
    classname = cl[d + 1:len(cl)]
    m = __import__(cl[0:d], globals(), locals(), [classname])
    return getattr(m, classname)


def import_model(clazzpath):
    """Will return the clazz defined by modul entry in the database of
    the given model. The clazzpath defines the base clazz which which
    defines the ID of the modul it belongs to.
    The function will first import the clazz and load the related modul
    entry for the model from the database. The we look for the clazzpath
    entry of the modul. If the moduls clazzpath is the same as the model
    clazzpath the return the imported model.
    If the clazzpath differs then import the model defined by the moduls
    clazzpath."""
    from ringo.model.modul import ModulItem
    orig_clazz = dynamic_import(clazzpath)
    # Load entry from the database for the given modul
    mid = orig_clazz._modul_id
    modul = DBSession.query(ModulItem).filter_by(id=mid).one()
    if modul.clazzpath == clazzpath:
        return orig_clazz
    else:
        # TODO: Is this code ever reached? (ti) <2014-02-25 23:07>
        return import_model(modul.clazzpath)


def get_item_modul(request, item):
    if hasattr(item, "_modul_id"):
        return _get_item_modul(request, item)
    else:
        return _get_item_modul(request, item.__class__)


def _get_item_modul(request, item):
    if not request:
        # FIXME: Ideally there is no need to call the methods without a
        # request and to get the request from the app globals. If this
        # code path is executed this is a sign the the caller is not
        # implemented in a clean way. Well, this is a known issue for
        # years and it currently doesn't seem to cause problems in the
        # real world. (ti) <2016-01-12 08:55>
        request = get_current_request()
    if not request or not request.cache_item_modul.get(item._modul_id):
        from ringo.model.modul import ModulItem
        factory = ModulItem.get_item_factory()
        if item._modul_id is None:
            # FIXME: Special case when loading fixtures for extensions.
            # As the id of an extension is set dynamically on
            # application startup the id is not yet present at time of
            # fixture loading. (ti) <2015-02-17 23:00>
            modul = factory.load(item.__tablename__, field="name")
        else:
            modul = factory.load(item._modul_id)
        if request:
            if not request.cache_item_modul.get(item._modul_id):
                request.cache_item_modul.set(item._modul_id, modul)
        else:
            return modul
    return request.cache_item_modul.get(item._modul_id)


def get_item_actions(request, item):
    """Returns a list of ActionItems which are available for given item
    or class.  If you want to add custom actions to the modul please
    overwrite this method.

    :item: Instance or class in the model.
    :returns: List of ActionItems.
    """
    modul = get_item_modul(request, item)
    return modul.actions


def get_action_routename(item, action, prefix=None):
    """Returns a string for the given action which can be used to get or
    build a route. If prefix is provided the the prefix will be
    prepended to the route name.

    :item: Instance or class in the model.
    :action: Name of the action.
    :prefix: Optional prefix of the route name.
    :returns: List of ActionItems.
    """
    routename = "%s-%s" % (item.__tablename__, action)
    if prefix:
        return "%s-%s" % (prefix, routename)
    return routename


def get_open_url(request, item):
    """Convinience method to the a URL to 'open' the item."""
    from ringo.lib.security import has_permission
    if has_permission("update", item, request):
        return request.route_path(get_action_routename(item, "update"),
                                  id=item.id)
    elif has_permission("read", item, request):
        return request.route_path(get_action_routename(item, "read"),
                                  id=item.id)
    return None


def get_action_url(request, item, action):
    """Return an URL object for the given item and action. If the item
    is an instance of object then we assume that we want to get the URL
    for the specific item. So we add the ID of the instance to the url.
    If the item is the class, then no ID is added (Create actions e.g.)

    :item: loaded instance or class of an item
    :action: string of the action
    :returns: URL instance
    """
    route_name = get_action_routename(item, action)
    if isinstance(item, object):
        return request.route_path(route_name, id=item.id)
    # TODO: Is this code ever reached. See testcase (ti) <2014-02-25 23:17>
    return request.route_path(route_name)


def get_saved_searches(request, name):
    """Returns a dictionary with the saved searches for the named overview.
    :request: Current request
    :name: Name of the overview
    :returns: Dictionary with the saved seaerches.
    """
    if not request.user:
        return {}
    searches_dic = request.user.settings.get('searches', {})
    if searches_dic:
        searches_dic_search = searches_dic.get(name, {})
        return searches_dic_search
    return {}


def get_modules(request, display):
    #  FIXME: Circular import (ti) <2015-05-11 21:52> 
    from ringo.lib.security import has_permission
    user_moduls = []
    # The modules has been load already and are cached. So get them from
    # the cache
    for modul in request.cache_item_modul.all().values():
        # Only show the modul if it matches the desired display location
        # and if the modul has an "list" action which usually is used as
        # entry point into a modul.
        if (modul.display == display and modul.has_action('list')):
            clazz = dynamic_import(modul.clazzpath)
            if has_permission('list', clazz, request):
                user_moduls.append(modul)
    return user_moduls



"""
Mixins can be used to add certain functionality to the items of a
module.  Mixins are used in multiple inheritance. The mixin ensures that
the item will have all needed fields in the database and provides the
proper interface to use the added functionality. Example::

        class Comment(BaseItem, Nested, Meta, Owned, Base):
            __tablename__ = 'comments'
            _modul_id = 99
            id = sa.Column(sa.Integer, primary_key=True)
            comment = sa.Column('comment', sa.Text)

            ...

The comment class in the example only defines the two fields `id` and
`comment`. But as it inherits from :ref:`mixin_nested`, :ref:`mixin_meta` and
:ref:`mixin_owned` it also will have date fields with the creation and date of
the last update, references to the user and group which ownes the Comment.
Further the 'Nested' mixin will ensure the comments can reference each other
to be able to build a hierarchy structure (e.g Threads in the example of the
comments).

.. note::
   The Comment Mixin has been removed from ringo and converted in the
   the ringo comment extension.

.. important::

    As most of the mixins will add additional tables and database fields
    to your item it is needed to migrate your database to the new model.
    See :ref:`alembic_migration` section for more information.
"""
import datetime
import json
import logging
from sqlalchemy.ext.declarative import declared_attr

from sqlalchemy import (
    Column,
    Text,
    String,
    Integer,
    DateTime,
    ForeignKey,
    Table
)

from sqlalchemy.orm import (
    relationship,
    backref
)

from ringo.model import Base
from ringo.lib.helpers import get_raw_value
from ringo.lib.alchemy import get_columns_from_instance

log = logging.getLogger(__name__)


class Mixin(object):
    """Base mixin class"""

    @classmethod
    def get_mixin_actions(cls):
        """Returns a list of actions specific for the mixin. Overwrite
        this method if you want to add custom actions for your mixin.

        :returns: List of ActionItems
        """
        return []


class StateMixin(object):
    """Mixin to add one or more Statemachines to an item.  The
    statemachines are stored in a internal '_statemachines' dictionary.
    The current state is stored as integer value per item. This field
    must be created manually.  The name of the field which stores the
    value for the current state must be the keyname of the
    '_statemachines' dictionary.

    Example Mixin usage::

        class FooMixin(StateMixin):
            # Mixin inherited from the StateMixin to add the Foobar
            # state machine

            # Attach the statemachines to an internal dictionary
            _statemachines = {'foo_state_id': FooStatemachine}

            # Configue a field in the model which saves the current
            # state per state machine
            foo_state_id = sa.Column(sa.Integer, default=1)

            # Optional. Create a property to access the statemachine
            # like an attribute. This gets usefull if you want to access
            # the state in overview lists.
            @property
            def foo_state(self):
                state = self.get_statemachine('foo_state_id')
                return state.get_state()
    """

    """Mapping of statemachines to attributes. The key in the dictionary
    must be the name of the field which stores the integer value of the
    current state of the statemachine."""
    _statemachines = {}

    @classmethod
    def list_statemachines(cls):
        """Returns a list keys of configured statemachines"""
        return cls._statemachines.keys()

    def change_state(self, request, key, old_state_id, new_state_id):
        """Changes the state of the given statemachine from old to new.
        The statemachine is identified by the key in the _statemachines
        dictionary"""
        log.debug("%s -> %s" % (old_state_id, new_state_id))
        sm = self.get_statemachine(key, old_state_id, request)
        sm.set_state(new_state_id)
        # clear cached statemachines
        setattr(self, '_cache_statemachines', {})

    def get_statemachine(self, key, state_id=None, request=None):
        """Returns a statemachine instance for the given key

        :key: Name of the key of the statemachine
        :state_id: initial state of the statemachine
        :returns: Statemachine instance

        """
        if not hasattr(self, '_cache_statemachines'):
            cache = {}
        else:
            cache = getattr(self, '_cache_statemachines')
        if key not in cache:
            cache[key] = self._statemachines[key](self, key, state_id, request)
            setattr(self, '_cache_statemachines', cache)
        else:
            # As the statemachine is cached we need to overwrite the
            # internale _request parameter to have the current request
            # available in the state.
            cache[key]._request = request
        return cache[key]


class Meta(object):
    """Mixin to add a created and a updated datefield to items. The
    updated datefield will be updated on every update of the item with
    the datetime of the update. Date will be saved in UTC"""
    created = Column(DateTime, default=datetime.datetime.utcnow)
    updated = Column(DateTime, default=datetime.datetime.utcnow)

    @classmethod
    def update_handler(cls, request, item):
        """Will update the updated attribute to the current datetime.
        The mapper and the target parameter will be the item which
        iherits this meta mixin.

        :request: Current request
        :item: Item handled in the update.

        """
        item.updated = datetime.datetime.utcnow()


class Blobform(object):
    """Mixin to add a data fields to store form data as JSON in a single
    field. Further the fid references the form definiton. The mixin will
    overwrite the way how to get the form definiton and how to get
    values from the item."""
    data = Column(Text, default="{}")

    @declared_attr
    def fid(cls):
        return Column(Integer, ForeignKey('forms.id'))

    @declared_attr
    def form(cls):
        form = relationship("Form", uselist=False)
        return form

    def __getattr__(self, name):
        """This function tries to get the given attribute of the item if
        it can not be found using the usual way to get attributes. In
        this case we will split the attribute name by "." and try to get
        the attribute along the "." separated attribute name."""
        data = getattr(self, 'data')
        if data:
            json_data = json.loads(getattr(self, 'data'))
            if name in json_data:
                return json_data[name]
        return get_raw_value(self, name)

    def set_values(self, values):
        """Will set the given values into Blobform items. This function
        overwrites the default behavior of the BaseItem and takes care
        that the data will be saved in the data attribute as JSON
        string."""
        json_data = json.loads(self.get_values().get('data') or "{}")
        columns = get_columns_from_instance(self, True)
        for key, value in values.iteritems():
            # Ignore private form fields
            if key.startswith('_'):
                continue
            if key in columns:
                log.debug("Setting value '%s' for '%s' in DB" % (value, key))
                setattr(self, key, value)
            else:
                if isinstance(value, datetime.date):
                    value = str(value)
                log.debug("Setting value '%s' for '%s' in JSON" % (value, key))
                json_data[key] = value
        setattr(self, 'data', json.dumps(json_data))


class Version(Base):
    __tablename__ = 'versions'
    id = Column(Integer, primary_key=True)
    values = Column(Text, nullable=False)
    author = Column(String, nullable=False)
    date = Column(DateTime, nullable=False)


class Versioned(object):
    """Mixin to add version functionallity to a modul. This mixin is
    used to store different "versions" of an item. On each update of
    the item, the serialized values of the item will be saved in the
    version table. Item modul will have a "versions" relationship
    containing all versions of the values for this item."""

    @declared_attr
    def versions(cls):
        tbl_name = "nm_%s_versions" % cls.__name__.lower()
        nm_table = Table(tbl_name, Base.metadata,
                         Column('iid', Integer, ForeignKey(cls.id)),
                         Column('vid', Integer, ForeignKey("versions.id")))
        versions = relationship(Version, secondary=nm_table, cascade="all")
        return versions

    @classmethod
    def create_handler(cls, request, item):
        cls.update_handler(request, item)

    @classmethod
    def update_handler(cls, request, item):
        """Will add the serialized values of the item into the version
        table.

        :request: Current request
        :item: Item handled in the update.

        """
        version = Version()
        item_values = item.get_values(serialized=True)
        version.values = json.dumps(item_values)
        version.author = request.user.login
        version.date = datetime.datetime.utcnow()
        item.versions.append(version)

    def _get_version(self, author, id):
        if not author and not id:
            return self.versions[-2]
        # iterate over the revsersed versions ignoring the last one
        for version in self.versions[-2::-1]:
            if id and version.id == id:
                return version
            elif author and version.author == author:
                return version
        return self.versions[0]

    def get_previous_values(self, author=None, id=None):
        """Returns a dictionary parsed from the last log entry of the
        item containinge the previous values for each field if it has
        been changed."""
        if len(self.versions) < 1:
            return {}
        else:
            pvalues = {}
            last = self._get_version(author, id)
            try:
                values = json.loads(last.values)
                for field in values:
                    if field == "data" and isinstance(self, Blobform):
                        blobdata = json.loads(values[field])
                        for blobfield in blobdata:
                            pvalues[blobfield] = blobdata[blobfield]
                    pvalues[field] = values[field]
            except:
                log.warning(("Could not build previous values dict. "
                             "Maybe old log format?"))
        return pvalues


class Owned(object):
    """Mixin to add references to a user and a usergroup. This
    references are used to build some kind of ownership of the item. The
    ownership is used from the permission system.

    It is posible to configure inhertinace of the owner and group from
    a given parent element. This information is used only while creating
    new instances of the modul. If configured, the default group and
    owner information will be overwritten. This is done at the very end
    of the creation process. See ''save'' method of the BaseItem. You
    can configure the inhertiance by setting the name of the relation to
    the parent item in the ''_inherit_gid'' and ''_inherit_uid'' class
    variable.
    """

    _inherit_gid = None
    """Variable to configure a inheritance of the gid. The variable
    should be the name of the relation to the parent element from which
    the gid will be taken"""
    _inherit_uid = None
    """Variable to configure a inheritance of the uid. The variable
    should be the name of the relation to the parent element from which
    the uid will be taken"""
    _owner_cascades = None
    """Variable to configure the behavior of the relation to the parent
    item if the owner of an itme is deleted. Defaults to 'save-update,
    merge'.  See SQLAlchemy documentation for more details. This means a
    user can not be deleted if he still owns items. Set this variable to
    'all, delete' of you want to delete all items owned by the user
    too."""

    @declared_attr
    def uid(cls):
        fk_name = "fk_%s_%s_%s" % (cls.__tablename__, 'uid', 'users')
        return Column(Integer, ForeignKey('users.id',
                                          use_alter=True,
                                          name=fk_name))

    @declared_attr
    def owner(cls):
        # Defining cascading and backrefs on Userclass does not work and
        # seems to be senseless anyway.
        if cls.__name__ == "User":
            return relationship("User",
                                primaryjoin="User.id==%s.uid" % cls.__name__)
        else:
            # The backref is private and only used to place the cascading
            # behaviour.
            backref_name = "__%s_item" % cls.__name__.lower()
            return relationship("User",
                                primaryjoin="User.id==%s.uid" % cls.__name__,
                                backref=backref(backref_name,
                                                cascade=cls._owner_cascades))

    @declared_attr
    def gid(cls):
        fk_name = "fk_%s_%s_%s" % (cls.__tablename__, 'gid', 'usergroups')
        return Column(Integer, ForeignKey('usergroups.id',
                                          use_alter=True,
                                          name=fk_name))

    @declared_attr
    def group(cls):
        return relationship("Usergroup",
                            primaryjoin="Usergroup.id==%s.gid" % cls.__name__)

    def is_owner(self, user):
        """Returns true if the Item is owned by the given user."""
        return user.id == self.uid


class Nested(object):
    """Mixin to make nested (self-reference) Items possible. Each item
    can have a parent item and many children. The class will add two
    relation attribute to the inheriting class. The parent item is
    available under the *parent* attribute. The children items are
    available under the *children* attribute."""

    @declared_attr
    def parent_id(cls):
        name = "%s.id" % cls.__tablename__
        return Column(Integer, ForeignKey(name))

    @declared_attr
    def children(cls):
        name = cls.__name__
        join = "%s.id==%s.parent_id" % (name, name)
        return relationship(name,
                            primaryjoin=join,
                            backref=backref('parent', remote_side=[cls.id]),
                            cascade="all")

    def get_parents(self):
        """Return a list of all parents of the current item
        :returns: List of BaseItems

        """
        parents = []
        if not self.parent:
            return parents
        else:
            parents.append(self.parent)
            parents.extend(self.parent.get_parents())
        return parents

    def get_children(self):
        """Returns a list of all children und subchildren
        :returns: List of BaseItems

        """
        childs = []
        for child in self.children:
            childs.append(child)
            childs.extend(child.get_children())
        return childs



import logging
import shutil
import os
import time
from sqlalchemy import engine_from_config
import transaction

from invoke import run
from alembic.config import Config
from alembic import command
from pyramid.paster import (
    get_appsettings,
    setup_logging,
)
from ringo.lib.sql import DBSession, NTDBSession, setup_db_session
from ringo.lib.helpers import get_app_location, dynamic_import
from ringo.lib.imexport import (
    JSONExporter, JSONImporter,
    CSVExporter, CSVImporter
)
from ringo.model.modul import ModulItem

log = logging.getLogger(__name__)


def get_appsettings_(config_file):
    settings = get_appsettings(config_file)
    databaseurl = os.environ.get('DATABASE_URL')
    if databaseurl:
        settings['sqlalchemy.url'] = databaseurl
    return settings


def is_locked(filepath):
    """Checks if a file is locked by opening it in append mode.
    If no exception thrown, then the file is not locked.
    """
    locked = None
    file_object = None
    if os.path.exists(filepath):
        try:
            #print "Trying to open %s." % filepath
            buffer_size = 8
            # Opening file in append mode and read the first 8 characters.
            file_object = open(filepath, 'a', buffer_size)
            if file_object:
                #print "%s is not locked." % filepath
                locked = False
        except IOError, message:
            print "File is locked (unable to open in append mode). %s." % \
                  message
            locked = True
        finally:
            if file_object:
                file_object.close()
                #print "%s closed." % filepath
    else:
        print "%s not found." % filepath
    return locked


def get_last_revision_file(args):
    """Will return the name of the latest revision file of the application"""
    script_path = []
    script_path.append(get_app_location(args.app))
    script_path.append(args.app)
    script_path.append("alembic")
    script_path.append("versions")
    cwd = os.getcwd()
    os.chdir(os.path.join(*script_path))
    script_path.append(max([fn for fn in os.listdir(".") if fn.endswith("py")],
                       key=os.path.getctime))
    os.chdir(cwd)
    return os.path.join(*script_path)

def get_engine(config_file):
    setup_logging(config_file)
    settings = get_appsettings_(config_file)
    engine = engine_from_config(settings, 'sqlalchemy.')
    setup_db_session(engine)
    return engine

def get_session(config_file, transactional=True):
    """Will return a database session based on the application
    configuration"""
    engine = get_engine(config_file)
    if transactional:
        DBSession.configure(bind=engine)
        return DBSession
    else:
        NTDBSession.configure(bind=engine)
        return NTDBSession


def copy_initial_migration_scripts(args):
    """Will copy the initial db migration scripts into the alembic
    versions folder of the application.
    """
    dst_path = []
    dst_path.append(get_app_location(args.app))
    dst_path.append(args.app)
    dst_path.append("alembic")
    dst_path.append("versions")
    src_path = []
    src_path.append(get_app_location(args.base))
    src_path.append(args.base)
    src_path.append("alembic")
    src_path.append("versions")

    src = os.path.join(*src_path)
    dst = os.path.join(*dst_path)
    dst_files = os.listdir(dst)
    src_files = os.listdir(src)
    # Only copy the initial files if the directory is empty and src and
    # dst isn't the same directory (Happens when installing ringo
    # itself).
    if (src == dst) or (len(dst_files) > 1):
        return
    for file_name in src_files:
        full_file_name = os.path.join(src, file_name)
        if (os.path.isfile(full_file_name)):
            shutil.copy(full_file_name, dst)


def create_new_revision(args, msg=None):
    """Will create a new autogenerated revision file. An returns the
    path of the new generated file."""
    cfg = get_alembic_config(args)
    command.revision(cfg, message=msg, autogenerate=True)
    revision_file = get_last_revision_file(args)
    while is_locked(revision_file):
        time.sleep(1)
    return revision_file


def get_alembic_config(args, app=None):
    """Return a alembic configuration

    :app: Name of the application
    :returns: Alembic configuration

    """

    config_path = []
    config_path.append(get_app_location(args.app))
    config_path.append(args.app)
    config_path.append("alembic")
    alembic_dir = os.path.join(*config_path)
    config_path.append("alembic.ini")
    cfg = Config(os.path.join(*config_path))
    if args.config:
        app_config = get_appsettings_(args.config)
        cfg.set_main_option("sqlalchemy.url",
                            app_config.get('sqlalchemy.url'))
        cfg.set_main_option("script_location",
                            alembic_dir)
        cfg.set_main_option("app_config",
                            args.config)
    return cfg


def handle_db_init_command(args):
    copy_initial_migration_scripts(args)
    handle_db_upgrade_command(args)


def handle_db_upgrade_command(args):
    cfg = get_alembic_config(args)
    command.upgrade(cfg, "head")
    if cfg.get_main_option("sqlalchemy.url").find("postgres") > -1:
    	handle_db_fixsequence_command(args)

def handle_db_downgrade_command(args):
    cfg = get_alembic_config(args)
    command.downgrade(cfg, "-1")
    handle_db_fixsequence_command(args)

def handle_db_revision_command(args):
    path = create_new_revision(args)
    print "New migration script created: %s" % path

def handle_db_savedata_command(args):
    path = []
    path.append(args.config)
    session = get_session(os.path.join(*path))
    modul_clazzpath = session.query(ModulItem).filter(ModulItem.name == args.modul).all()[0].clazzpath
    modul = dynamic_import(modul_clazzpath)
    if args.format == "json":
        exporter = JSONExporter(modul, serialized=False,
                                relations=args.include_relations)
    else:
        exporter = CSVExporter(modul, serialized=False,
                               relations=args.include_relations)
    print exporter.perform(session.query(modul)
                          .order_by(modul.id)
                          .all())

def handle_db_loaddata_command(args):
    path = []
    path.append(args.config)
    session = get_session(os.path.join(*path))
    importer = get_importer(session, args.modul, args.format)
    with open(args.fixture) as f:
        data = f.read()
        items, created, updated = do_import(session,
                                            importer, data,
                                            (not args.loadbyid))

    try:
        transaction.commit()
        print "Updated %s items, Created %s items" % (updated, created)
    except Exception as e:
        print str(e)
        print "Loading data failed!"


def get_importer(session, modulname, fmt):
    modul_clazzpath = session.query(ModulItem).filter(ModulItem.name == modulname).all()[0].clazzpath
    modul = dynamic_import(modul_clazzpath)
    if fmt == "json":
        return JSONImporter(modul, session)
    else:
        return CSVImporter(modul, session)


def do_import(session, importer, data, use_uuid=True):
    items = []
    updated = 0
    created = 0
    items = importer.perform(data, use_uuid=use_uuid)
    for item, action in items:
        # Add all new items to the session
        if action.find("CREATE") > -1:
            session.add(item)
            created += 1
        else:
            updated += 1
    return items, created, updated


def handle_db_uuid_command(args):
    path = []
    path.append(args.config)
    session = get_session(os.path.join(*path))
    modul_clazzpath = session.query(ModulItem).filter(ModulItem.name == args.modul).all()[0].clazzpath
    modul = dynamic_import(modul_clazzpath)
    updated = 0
    created = 0
    for item in session.query(modul).all():
        if item.uuid:
            if args.missing_only:
                continue
            else:
                item.reset_uuid()
                updated += 1
        else:
            item.reset_uuid()
            created += 1
    try:
        transaction.commit()
        print "Updated %s items, Created %s items" % (updated, created)
    except:
        print "Loading data failed!"

def _get_user_id_function():
    out = []
    out.append("CREATE OR REPLACE FUNCTION uid() RETURNS integer")
    out.append("LANGUAGE 'plpgsql' STABLE")
    out.append("AS $$")
    out.append("  DECLARE")
    out.append("  uid integer;")
    out.append("  BEGIN")
    out.append("    SELECT INTO uid id FROM users WHERE login = session_user;")
    out.append("    RETURN uid;")
    out.append("  END;")
    out.append("$$;")
    return "\n".join(out)


def handle_db_restrict_command(args):
    """Will return a SQL statement to restrict the access to the given
    tables by revoking all priviledges to the table in the public
    scheme. Instead of this a view with the same name as the table in
    the restricted view will be defined which only returns datasets
    which the current user owns or is member of the items group."""
    out = []
    out.append("-- Execute this as user postgres")
    out.append("CREATE SCHEMA IF NOT EXISTS restriced;")
    out.append(_get_user_id_function())
    out.append("REVOKE ALL ON TABLE public.%ss FROM PUBLIC;" % args.modul)
    out.append("CREATE OR REPLACE VIEW %ss AS " % args.modul)
    out.append("SELECT * FROM public.%ss WHERE " % args.modul)
    out.append("uid = uid()")
    out.append("OR gid IN ( SELECT gid from nm_user_usergroups where uid = uid() ) ")
    out.append("OR uid() IN ( select uid from nm_user_roles where rid = 1 );")
    out.append("GRANT ALL ON TABLE restricted.%ss TO PUBLIC;" % args.modul)
    out.append("-- Do not to forget to change the default searchpath to '$user', restricted, public")
    print "\n".join(out)


def handle_db_unrestrict_command(args):
    """Will return a SQL statement to unrestrict the access to the given
    tables by granting all priviledges to the table in the public
    scheme. The restricted view will be dropped."""
    out = []
    out.append("-- Execute this as user postgres")
    out.append("GRANT ALL ON TABLE public.%ss TO PUBLIC;" % args.modul)
    out.append("DROP VIEW IF EXISTS %ss;" % (args.modul))
    print "\n".join(out)

def handle_db_fixsequence_command(args):
    """Will fix the sequences of the primary keys in all tables of a
    PostgresSQL Database. For each table in the database the function
    will determine the highest value for the id and set the nextval to
    the max+1. Background. When inserting items per fixtures the fixture
    may contain an id for the new item. In this case the id will be set
    in the database but the internal sequence counter is not updated."""

    # Load the SQL Script to fix the sequence
    sql_path = []
    sql_path.append(get_app_location("ringo"))
    sql_path.append("ringo/scripts/generate_fix_sequence_stmnts.sql")
    with open(os.path.join(*sql_path)) as sqlfile:
        sql = sqlfile.read()

    # Get DB connection and apply loaded SQL statements.
    path = []
    path.append(args.config)
    engine = get_engine(os.path.join(*path))
    conn = engine.connect()
    result = conn.execute(sql)
    print "Fixing sequences ... ",
    for r in result:
        conn.execute(r[0])
    conn.close()
    print "OK"



"""Added user_logins table to store login attemps

Revision ID: 38eb5f63a8a4
Revises: ac95cbd0fce
Create Date: 2015-11-17 10:56:40.862774

"""

# revision identifiers, used by Alembic.
revision = '38eb5f63a8a4'
down_revision = 'ac95cbd0fce'

from alembic import op
import sqlalchemy as sa


UPGRADE = """"""
DOWNGRADE = """"""


def iter_statements(stmts):
    for st in [x for x in stmts.split('\n') if x]:
        op.execute(st)


def upgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.create_table('user_logins',
    sa.Column('id', sa.Integer(), nullable=False),
    sa.Column('datetime', sa.DateTime(), nullable=True),
    sa.Column('success', sa.Boolean(), nullable=True),
    sa.Column('uid', sa.Integer(), nullable=True),
    sa.ForeignKeyConstraint(['uid'], ['users.id'], ),
    sa.PrimaryKeyConstraint('id')
    )
    ### end Alembic commands ###
    iter_statements(UPGRADE)


def downgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.drop_table('user_logins')
    ### end Alembic commands ###
    iter_statements(DOWNGRADE)



import logging
from pyramid.httpexceptions import HTTPFound

from ringo.lib.imexport import (
    JSONImporter,
    CSVImporter
)
from ringo.lib.renderer import (
    ImportDialogRenderer,
    ErrorDialogRenderer
)
from ringo.lib.sql.cache import invalidate_cache
from ringo.views.request import (
    handle_params,
    handle_event,
    handle_history,
    is_confirmed
)

log = logging.getLogger(__name__)

def _import(request):
    """Will read the import file from the request and create or update
    the items in the import file. Finally a list of items will be
    returned. The returned list is a list of tuples. The first element
    in the tuple is the item, the second element is a string with the
    operation (create, update) which was done during import for the
    item.

    :request: Request with importfile and importformat
    :returns: List of items.

    """
    clazz = request.context.__model__
    try:
        request.POST.get('file').file.seek(0)
    except AttributeError:
        # This is triggered when no file has been selected for import
        raise AttributeError
    importfile = request.POST.get('file').file.read()
    if request.POST.get('format') == 'json':
        importer = JSONImporter(clazz, request.db)
    elif request.POST.get('format') == 'csv':
        importer = CSVImporter(clazz)
    return importer.perform(importfile, request.user, request.translate)


def _handle_save(request, items, callback):
    """This function will actually save the imported items. It iterates
    over the imported items and tries to

    * Save the item
    * Call the callback function
    * Call the import handler

    This function returns a list of tuples. The first element of the
    tuple is the element. The second element is the operation done in
    the initial import and the last element is a boolean flag indicating
    if the saving has succeeded.

    :request: Current request
    :items: List of imported items
    :callback: Callback function
    :returns: List of imported items

    """
    imported_items = []
    for item in items:
        item, operation = item[0], item[1]
        try:
            item.save(item.get_values(), request)
            if callback:
                item = callback(request, item)
            # handle update events
            handle_event(request, item, 'import')
            imported_items.append((item, operation, True))
        except:
            imported_items.append((item, operation, False))
    return imported_items


def _handle_redirect(request):
    """If there is a backurl in the session a redirect will be returned.
    Else None.

    :request: Current request
    :returns: Redirect or None

    """
    clazz = request.context.__model__
    backurl = request.session.get('%s.backurl' % clazz)
    if backurl:
        # Redirect to the configured backurl.
        del request.session['%s.backurl' % clazz]
        request.session.save()
        return HTTPFound(location=backurl)


def import_(request, callback=None):
    handle_history(request)
    handle_params(request)

    clazz = request.context.__model__
    _ = request.translate
    renderer = ImportDialogRenderer(request, clazz)
    imported_items = []
    form = renderer.form
    if (request.method == 'POST'
       and is_confirmed(request)
       and form.validate(request.params)):
        try:
            items = _import(request)
        except (ValueError, AttributeError) as e:
            err_title = _("Import failed")
            err_msg = _("Bad news! The import could not be finished and "
                        "returns with an error."
                        "No data has been modified in this operation and no "
                        "items has been imported or updated. "
                        "The last message we heard from the importer was: %s"
                        ) % e
            renderer = ErrorDialogRenderer(request, err_title, err_msg)
            rvalue = {}
            ok_url = request.session['history'].pop(2)
            rvalue['dialog'] = renderer.render(ok_url)
            rvalue['clazz'] = clazz
            return rvalue

        imported_items = _handle_save(request, items, callback)
        invalidate_cache()
        redirect = _handle_redirect(request)
        if redirect:
            return redirect
    rvalue = {}
    rvalue['dialog'] = renderer.render(imported_items)
    rvalue['clazz'] = clazz
    return rvalue



import logging
import sqlalchemy as sa
from pyramid.httpexceptions import HTTPFound
from ringo.lib.sql.cache import invalidate_cache
from ringo.lib.renderer import ConfirmDialogRenderer, InfoDialogRenderer
from ringo.lib.helpers import (
    get_action_routename,
    get_item_modul
)
from ringo.views.response import JSONResponse
from ringo.views.request import (
    handle_params,
    handle_history,
    is_confirmed,
    get_item_from_request
)
from ringo.views.base.list_ import set_bundle_action_handler

log = logging.getLogger(__name__)


def _handle_redirect(request):
    """Will return a redirect.

    :request: Current request
    :returns: Redirect

    """
    clazz = request.context.__model__
    backurl = request.session.get('%s.backurl' % clazz)
    if backurl:
        # Redirect to the configured backurl.
        del request.session['%s.backurl' % clazz]
        request.session.save()
        return HTTPFound(location=backurl)
    else:
        # Redirect to the list view.
        route_name = get_action_routename(clazz, 'list')
        url = request.route_path(route_name)
        return HTTPFound(location=url)


def _handle_delete_request(request, items, callback):
    clazz = request.context.__model__
    _ = request.translate
    if request.method == 'POST' and is_confirmed(request):
        item_label = get_item_modul(request, clazz).get_label(plural=True)
        item_label_log = get_item_modul(request, clazz).get_label()
        mapping = {'item_type': item_label, 'num': len(items)}
        for item in items:
            if callback:
                item = callback(request, item)
            request.db.delete(item)
        # Invalidate cache
        invalidate_cache()
        try:
            request.db.flush()
        except (sa.exc.CircularDependencyError, sa.exc.IntegrityError) as e:
            mapping["error"] = e.message.decode("utf-8")
            title = _("Can not delete ${item_type} items.",
                      mapping=mapping)
            body = _("There has been an integrity error which prevents "
                     "the request to be fulfilled. There are still "
                     "depended items on the item to be deleted. Please "
                     "remove all depended relations to this item before "
                     "deleting it and try again. Hint: ${error}",
                     mapping=mapping)
            request.db.rollback()
            renderer = InfoDialogRenderer(request, title, body)
            rvalue = {}
            ok_url = request.session['history'].pop(2)
            rvalue['dialog'] = renderer.render(ok_url)
            return rvalue

        msg = _('Deleted ${num} ${item_type} successfully.', mapping=mapping)
        log_msg = u'User {user.login} deleted {item_label} {item.id}' \
            .format(item_label=item_label, item=item, user=request.user)
        log.info(log_msg)
        request.session.flash(msg, 'success')
        # Handle redirect after success.
        return _handle_redirect(request)
    else:
        renderer = ConfirmDialogRenderer(request, clazz, 'delete')
        rvalue = {}
        rvalue['dialog'] = renderer.render(items)
        rvalue['clazz'] = clazz
        rvalue['item'] = items
        return rvalue


def delete(request, callback=None):
    item = get_item_from_request(request)
    handle_history(request)
    handle_params(request)
    return _handle_delete_request(request, [item], callback)


def rest_delete(request, callback=None):
    """Deletes an item of type clazz. The item is deleted based on the
    unique id value provided in the matchtict object in the current
    DELETE request. The data will be deleted without any futher confirmation!

    :clazz: Class of item to delete
    :request: Current request
    :returns: JSON object.
    """
    item = get_item_from_request(request)
    request.db.delete(item)
    return JSONResponse(True, item)

set_bundle_action_handler("delete", _handle_delete_request)



"""Modul to handle requests."""
import logging
import urllib
import urlparse
import transaction
from pyramid.httpexceptions import HTTPFound
from formbar.form import Validator
from ringo.lib.security import (
    has_permission,
    ValueChecker
)
from ringo.lib.helpers import import_model, get_action_routename, literal
from ringo.lib.history import History
from ringo.lib.sql.cache import invalidate_cache
from ringo.views.helpers import (
    get_item_from_request,
    get_ownership_form,
    get_item_form,
    get_item_modul,
    get_current_form_page,
    set_current_form_page,
    get_next_form_page
)

log = logging.getLogger(__name__)


def form_has_errors(field, data, context):
    """Simple validation callback which returns True if the form has not
    errors an validation. Please make sure that this validators gets
    called as last validator of all validators get the final result of
    the validation."""
    # context is the current formbar form.
    return not context.has_errors()


def encode_unicode_dict(unicodedict, encoding="utf-8"):
    bytedict = {}
    for key in unicodedict:
        if isinstance(unicodedict[key], unicode):
            bytedict[key] = unicodedict[key].encode(encoding)
        elif isinstance(unicodedict[key], dict):
            bytedict[key] = encode_unicode_dict(unicodedict[key])
        else:
            bytedict[key] = unicodedict[key]
    return bytedict


def decode_bytestring_dict(bytedict, encoding="utf-8"):
    unicodedict = {}
    for key in bytedict:
        if isinstance(bytedict[key], str):
            unicodedict[key] = bytedict[key].decode(encoding)
        elif isinstance(bytedict[key], dict):
            unicodedict[key] = decode_bytestring_dict(bytedict[key])
        else:
            unicodedict[key] = bytedict[key]
    return unicodedict


def encode_values(values):
    """Returns a string with encode the values in the given dictionary.

    :values: dictionary with key values pairs
    :returns: String key1:value1,key2:value2...

    """
    # Because urlencode can not handle unicode strings we encode the
    # whole dictionary into utf8 bytestrings first.
    return urllib.urlencode(encode_unicode_dict(values))


def decode_values(encoded):
    """Returns a dictionay with decoded values in the string. See
    encode_values function.

    :encoded : String key1:value1,key2:value2...
    :returns: Dictionary with key values pairs
    """
    # We convert the encoded querystring into a bystring to enforce that
    # parse_pq returns a dictionary which can be later decoded using
    # decode_bystring_dict. If we use the encoded string directly the
    # returned dicionary would contain bytestring as unicode. e.g
    # u'M\xc3\xbcller' which can't be decoded later.
    encoded = str(encoded)

    # Now convert the query string into a dictionary with UTF-8 encoded
    # bytestring values.
    values = urlparse.parse_qs(encoded)
    for key in values:
        values[key] = values[key][0]
    # Finally convert this dictionary back into a unicode dictionary
    return decode_bytestring_dict(values)


def is_confirmed(request):
    """Returns True id the request is confirmed"""
    return request.params.get('confirmed') == "1"


def handle_event(request, item, event):
    """Will call the event listeners for the given event on every base
    class of the given item."""
    for class_ in item.__class__.__bases__:
        if hasattr(class_, event + '_handler'):
            handler = getattr(class_, event + '_handler')
            handler(request, item)


def handle_callback(request, callback, item=None):
    """Will call the given callback

    :request: Current request
    :callback: Callable function or list of callable functions
    :item: item for which the callback will be called.
    :returns: item

    """
    if not item:
        item = get_item_from_request(request)
    if isinstance(callback, list):
        for cb in callback:
            item = cb(request, item)
    elif callback:
        item = callback(request, item)
    return item


def get_relation_item(request):
    clazz = request.context.__model__
    addrelation = request.session.get('%s.addrelation' % clazz)
    if not addrelation:
        addrelation = request.params.get("addrelation")
        if not addrelation:
            return None
    rrel, rclazz, rid = addrelation.split(':')
    parent = import_model(rclazz)
    factory = parent.get_item_factory()
    item = factory.load(rid, db=request.db)
    return item, rrel



def handle_add_relation(request, item):
    """Handle linking of the new item to antoher relation. The relation
    was provided as GET parameter in the current request and is now
    saved in the session.

    :request: Current request
    :item: new item with should be linked

    """
    relation_tuple = get_relation_item(request)
    if relation_tuple:
        rrel = relation_tuple[1]
        pitem = relation_tuple[0]
        clazz = request.context.__model__
        log.debug('Linking %s to %s in %s' % (item, pitem, rrel))
        tmpattr = getattr(pitem, rrel)
        tmpattr.append(item)
        # Delete value from session after the relation has been added
        del request.session['%s.addrelation' % clazz]
    else:
        return item
    request.session.save()


def handle_caching(request):
    """Will handle invalidation of the cache

    :request: TODO

    """
    # Invalidate cache
    invalidate_cache()
    clazz = request.context.__model__
    if request.session.get('%s.form' % clazz):
        del request.session['%s.form' % clazz]
    request.session.save()


def handle_POST_request(form, request, callback, event="", renderers=None):
    """@todo: Docstring for handle_POST_request.

    :name: @todo
    :request: @todo
    :callback: @todo
    :renderers: @todo
    :event: Name of the event (update, create...) Used for the event handler
    :returns: True or False

    """
    _ = request.translate
    clazz = request.context.__model__
    item_label = get_item_modul(request, clazz).get_label()
    item = get_item_from_request(request)
    mapping = {'item_type': item_label, 'item': item}

    # Add a *special* validator to the form to trigger rendering a
    # permanent info pane at the top of the form in case of errors on
    # validation. This info has been added because users reported data
    # loss because of formbar/ringo default behaviour of not saving
    # anything in case of errors. Users seems to expect that the valid
    # part of the data has been saved. This info should make the user
    # aware of the fact that nothing has been saved in case of errors.
    error_message = _("The information contained errors. "
                      "<strong>All entries (including error-free) were not "
                      "saved!</strong> Please correct your entries in the "
                      "fields marked in red and resave.")
    form.add_validator(Validator(None, literal(error_message),
                                 callback=form_has_errors,
                                 context=form))

    # Begin a nested transaction. In case an error occours while saving
    # the data the nested transaction will be rolled back. The parent
    # session will be still ok.
    request.db.begin_nested()
    if form.validate(request.params) and "blobforms" not in request.params:
        checker = ValueChecker()
        try:
            if event == "create":
                try:
                    factory = clazz.get_item_factory(request)
                except TypeError:
                    # Old version of get_item_factory method which does
                    # not take an request parameter.
                    factory = clazz.get_item_factory()
                    factory._request = request

                checker.check(clazz, form.data, request)
                item = factory.create(request.user, form.data)
                item.save({}, request)
                request.context.item = item
                handle_add_relation(request, item)
            else:
                values = checker.check(clazz, form.data, request, item)
                item.save(values, request)
            handle_event(request, item, event)
            handle_callback(request, callback)
            handle_caching(request)

            if event == "create":
                msg = _('Created new ${item_type} successfully.',
                        mapping=mapping)
                log_msg = u'User {user.login} created {item_label} {item.id}'\
                    .format(item_label=item_label, item=item, user=request.user)
            else:
                msg = _('Edited ${item_type} "${item}" successfully.',
                        mapping=mapping)
                log_msg = u'User {user.login} edited {item_label} {item.id}'\
                    .format(item_label=item_label, item=item, user=request.user)
            log.info(log_msg)
            request.session.flash(msg, 'success')

            # Set next form page.
            if request.params.get("_submit") == "nextpage":
                table = clazz.__table__
                itemid = item.id
                page = get_next_form_page(form,
                                          get_current_form_page(clazz,
                                                                request))
                set_current_form_page(table, itemid, page, request)

            # In case all is ok merge the nested session.
            request.db.merge(item)
            return True
        except Exception as error:
            request.db.rollback()
            mapping['error'] = unicode(error.message)
            if event == "create":
                msg = _('Error while saving new '
                        '${item_type}: ${error}.', mapping=mapping)
            else:
                msg = _('Error while saving '
                        '${item_type} "${item}": ${error}.', mapping=mapping)
            log.exception(msg)
            request.session.flash(msg, 'critical')
            return False
    elif "blobforms" in request.params:
        pass
    else:
        request.db.rollback()
        if event == "create":
            msg = _('Error on validation new '
                    '${item_type}.', mapping=mapping)
        else:
            msg = _('Error on validation '
                    '${item_type} "${item}".', mapping=mapping)
        log.debug(msg)
        request.session.flash(msg, 'error')
    return False


def handle_redirect_on_success(request, backurl=None):
    """Will return a redirect. If there has been a saved "backurl" the
    redirect will on on this url. In all other cases the function will
    try to determine if the item in the request can be opened in edit
    mode or not. If the current user is allowed to open the item in
    edit mode then the update view is called. Else the read view is
    called.

    :request: Current request
    :backurl: Optional. Set backurl manually. This will overwrite
    backurls saved in the session. 
    :returns: Redirect
    """

    item = get_item_from_request(request)
    clazz = request.context.__model__
    backurl = backurl or request.session.get('%s.backurl' % clazz)
    if backurl:
        # Redirect to the configured backurl.
        if request.session.get('%s.backurl' % clazz):
            del request.session['%s.backurl' % clazz]
            request.session.save()
        return HTTPFound(location=backurl)
    else:
        # Handle redirect after success.
        # Check if the user is allowed to call the url after saving
        if has_permission("update", item, request):
            route_name = get_action_routename(item, 'update')
            url = request.route_path(route_name, id=item.id)
        else:
            route_name = get_action_routename(item, 'read')
            url = request.route_path(route_name, id=item.id)
        return HTTPFound(location=url)


def handle_history(request):
    history = request.session.get('history')
    if history is None:
        history = History([])
    history.push(request.url)
    request.session['history'] = history


def handle_params(request):
    """Handles varios sytem GET params comming with the request
    Known params are:

     * backurl: A url which should be called instead of the default
       action after the next request succeeds. The backurl will be saved
       in the session and stays there until it is deleted on a
       successfull request. So take care to delete it to not mess up
       with the application logic.
     * form: The name of a alternative form configuration which is
       used for the request.
     * values: A comma separated list of key/value pair. Key and value
       are separated with an ":"
     * addrelation: A ":" separated string 'relation:class:id' to identify that
       a item with id should be linked to the relation of this item.
    """
    clazz = request.context.__model__
    params = {}
    backurl = request.GET.get('backurl')
    if backurl:
        request.session['%s.backurl' % clazz] = backurl
        params['backurl'] = backurl
    values = request.GET.get('values')
    if values:
        params['values'] = {}
        values = decode_values(values)
        for key in values:
            params['values'][key] = values[key]
    form = request.GET.get('form')
    if form:
        #request.session['%s.form' % clazz] = form
        params['form'] = form
    relation = request.GET.get('addrelation')
    if relation:
        request.session['%s.addrelation' % clazz] = relation
        params['addrelation'] = relation
    request.session.save()
    return params


def get_return_value(request):
    """Will return a dictionary of values used as context in the
    templates. The dictionary will include:

    * clazz: clazz of the current request
    * item: item of the current request

    :request: Current request
    :returns: Dictionary with values as context
    """
    rvalues = {}
    clazz = request.context.__model__
    item = get_item_from_request(request)
    rvalues['clazz'] = clazz
    rvalues['item'] = item
    return rvalues
